<?php
namespace com\nestdesign\WpJobManagerApi2\Applications;
use com\nestdesign\WpJobManagerApi2\Main;

class Application {
  protected static $verbose = true;
  protected static $log_directory = null;
  public static function init() {
    add_action('init', function () {
      add_post_type_support(Main::currentPostType(), 'comments');
      if(class_exists('WP_Job_Manager_Post_Types')) {
        remove_action('job_manager_application_details_email', array(\WP_Job_Manager_Post_Types::instance(), 'application_details_email'));
        add_action('job_manager_application_details_email', '\com\nestdesign\WpJobManagerApi2\Applications\Application::showApplyForm');
      }
    });
  }
  public static function showApplyForm() {
    do_action(Main::ACTION_DISPLAY_APPLY_FORM);
  }
  protected static function shouldAttempNotify() {
    return true;
  }
  protected function submit_apply_notification($job, $data, $attachments = array(), $force_user_email = null) {
    static $fp_verbose;
    $apply_notification_url = get_post_meta($job->ID, Main::META_APPLICATION_URL, true);
    if(empty($apply_notification_url)) {
      return;
    }
    $data = array(
      'title' => $job->post_title,
      'job_id' => get_post_meta($job->ID, Main::META_REMOTE_ID, true),
      'internal_job_id' => $job->ID,
      'job_url' => get_permalink($job),
      'user_email' => !empty($force_user_email) ? $force_user_email : get_post_meta($job->ID, Main::META_APPLICATION, true),
      'website' => $_SERVER['HTTP_REFERER'],
      'data' => $data,
    );
    $i = 0;
    $application_json = get_option(Main::OPTION_ENABLE_APPLICATION_IN_JSON, false);
    if(!empty($application_json)) {
      foreach($attachments as $attachment) {
        $data['attachment'.($i++)] = array('filename' => $attachment->postname, 'content' => base64_encode(file_get_contents($attachment->name)));
      }
    } else {
      foreach($attachments as $attachment) {
        $data['attachment'.($i++)] = $attachment;
      }
    }
    $tracking_source_enabled = get_option(Main::OPTION_ENABLE_APPLICATION_SOURCE_TRACK, false);
    if(!empty($tracking_source_enabled)) {
      $data['source_url'] = 'N/A';
      if(!empty($_SESSION[Main::KEY_APPLICATION_SOURCE_TRACK])) {
        $data['source_url'] = $_SESSION[Main::KEY_APPLICATION_SOURCE_TRACK];
      } else if(isset($_COOKIE[Main::KEY_APPLICATION_SOURCE_TRACK])) {
        $data['source_url'] = $_COOKIE[Main::KEY_APPLICATION_SOURCE_TRACK];
      }
    }
    $ch = curl_init();
    if(!empty(self::$verbose)) {
      if(empty($fp_verbose)) {
        $fp_verbose = fopen (self::getLogDirectory()."/verbose_".date('Ymd-His').".log", "a+");
        fwrite($fp_verbose, var_export(array('data' => $data, 'req' => $_REQUEST, 'files' => $_FILES, 'server' => $_SERVER), true)."\r\n\r\n");
      }
      curl_setopt($ch, CURLOPT_VERBOSE, true);
      curl_setopt($ch, CURLOPT_STDERR, $fp_verbose);
    }
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_URL, $apply_notification_url);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_TIMEOUT, 10);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_USERAGENT, "YouRecruit");
    if($application_json) {
      curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
      curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
    } else {
      $post = self::fixCurlPost($data);
      if(!is_string($post))
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: multipart/form-data'));
      curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
    }
    $res = curl_exec($ch);
    $did_sent = true;
    wp_insert_comment(array('comment_post_ID' => $job->ID, 'comment_content' => 'Submit to Notify url '.($res ? 'suceeded' : 'failed'), 'comment_agent' => $_SERVER['USER_AGENT']));
    curl_close($ch);
  }
  static function getLogDirectory() {
    if(empty(self::$log_directory)) {
      if(function_exists('wp_upload_dir')) {
        $upload_dir = wp_upload_dir();
        self::$log_directory = $upload_dir['basedir'].'/api2';
        if(!is_dir(self::$log_directory))
          mkdir(self::$log_directory, 0777);
      } else {
        self::$log_directory = dirname(__FILE__).'/logs';
      }
    }
    return self::$log_directory;
  }
  protected static function fixCurlPost($data, $prefix = '', &$hasFile = false) {
    $out = array();
    if(is_array($data)) {
      foreach($data as $key => $value) {
        $k = !empty($prefix) ? $prefix.'['.$key.']' : $key;
        if(is_scalar($value)) {
          $out[$k] = $value;
        } else {
          $out += self::fixCurlPost($value, $k, $hasFile);
        }
      }
    } else if ($data instanceof \CURLFile) {
      $hasFile = true;
      return array($prefix => $data);
    } else {
      return array($prefix => $data);
    }
    if(empty($prefix) && $hasFile === false)
      return http_build_query($data);
    return $out;
  }
}
Application::init();
