<?php
namespace com\nestdesign\WpJobManagerApi2\Applications;
use com\nestdesign\WpJobManagerApi2\Main;

class GravityFormsApplications extends Application {
  public static function init() {
    add_action(Main::ACTION_DISPLAY_APPLY_FORM, '\com\nestdesign\WpJobManagerApi2\Applications\GravityFormsApplications::displayApplicationForm');
    add_filter('job_manager_settings', function ($settings) {
      $settings['wp_job_manager_api2'][1][] = array(
        'name'  => Main::OPTION_APPPLY_FORMS,
        'subname' => 'gravityforms',
        'label' => __( 'Apply with Gravity Form', 'wp-job-manager-api2' ),
        'type' => 'wp_job_manager_api2_apply_gravity_form',
        'desc' => __('Select the form to show on the Job Detail page', 'wp-job-manager-api2'),
      );
      return $settings;
    }, 11, 1);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_apply_gravity_form', function ($option, $attributes, $value, $placeholder = null) {
      $val = isset($value[$option['subname']]) ? $value[$option['subname']] : null;
      echo '<select name="'.htmlspecialchars($option['name'].'['.$option['subname'].'][id]', ENT_QUOTES).'">';
      $forms = \GFAPI::get_forms();
      if(!empty($forms)) {
        echo '<option value="">'.__('-- select --', 'wp-job-manager-api2').'</option>';
        foreach ( $forms as $form) {
          echo '<option value="'.$form['id'].'"'.((int)$val['id'] === $form['id'] ? ' selected="selected"' : '').'>'.$form['title'].'</option>';
        }
      } else {
        echo '<option value="">'.__('-- No Forms Found --', 'wp-job-manager-api2').'</option>';
      }
      echo '</select>';
      echo '<label><input type="checkbox"'.(!empty($val['hide_title']) ? ' checked="checked"' : '').' name="'.htmlspecialchars($option['name'].'['.$option['subname'].'][hide_title]', ENT_QUOTES).'" />'.__( 'Hide Title', 'wp-job-manager-api2' ).'</label>';
      echo '<label><input type="checkbox"'.(!empty($val['hide_description']) ? ' checked="checked"' : '').' name="'.htmlspecialchars($option['name'].'['.$option['subname'].'][hide_description]', ENT_QUOTES).'" />'.__( 'Hide Description', 'wp-job-manager-api2' ).'</label>';
      echo '<label><input type="checkbox"'.(!empty($val['use_ajax']) ? ' checked="checked"' : '').' name="'.htmlspecialchars($option['name'].'['.$option['subname'].'][use_ajax]', ENT_QUOTES).'" />'.__( 'Use Ajax', 'wp-job-manager-api2' ).'</label>';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_filter('gform_custom_merge_tags', function ($tags, $form_id, $fields, $element_id) {
      $tags[] = array(
				'tag'   => '{ref_key}',
				'label' => esc_html__( 'Job Reference Key', 'wp-job-manager-api2' )
			);
      $tags[] = array(
				'tag'   => '{post_title}',
				'label' => esc_html__( 'Job Post Title', 'wp-job-manager-api2' )
			);
      return $tags;
    }, 10, 4);
    add_filter('gform_pre_replace_merge_tags', function ($text, $form, $lead, $url_encode, $esc_html, $nl2br, $format) {
      if(strpos($text, '{') !== false) {
        $post_id = $lead['post_id'];
        if(empty($post_id) && !empty($lead['source_url'])) {
          $post_id = url_to_postid($lead['source_url']);
        }
        if(strpos($text, '{consultant_email}') !== false) {
          $email = get_post_meta($post_id, '_application', true);
          if(!empty($email)) {
            $text = str_replace('{consultant_email}', $email, $text);
          }
        }
        foreach(array('_ref_key') as $metakey) {
          if(strpos($text, '{'.trim($metakey, '_').'}') !== false) {
            $text = str_replace('{'.trim($metakey, '_').'}', get_post_meta($post_id, $metakey, true), $text);
          } else if (strpos($text, '{'.$metakey.'}') !== false) {
            $text = str_replace('{'.$metakey.'}', get_post_meta($post_id, $metakey, true), $text);
          }
        }
        $post = get_post($post_id);
        foreach(array('post_title') as $metakey) {
          if(strpos($text, '{'.$metakey.'}') !== false) {
            $text = str_replace('{'.$metakey.'}', $post->$metakey, $text);
          }
        }
      }
      return $text;
    }, 10, 7);
    add_action('gform_entry_created', function ($lead, $form) {
      $apply = get_option(Main::OPTION_APPPLY_FORMS);
      if(!empty($apply['gravityforms']['id']) && $form['id'] === (int)$apply['gravityforms']['id']) {
        self::handleLead($lead, $form);
      }
    }, 10, 2);
    add_filter('gform_notification', function ($notification, $form, $lead) {
      if(strpos($notification['to'], '{consultant_email}') !== false) {
        self::handleLead($lead, $form);
      }
      return $notification;
    }, 10, 3);
  }
  protected static function handleLead($lead, $form) {
    static $handledLeads = array();
    $post_id = $lead['post_id'];
    if(empty($post_id) && !empty($lead['source_url'])) {
      $post_id = url_to_postid($lead['source_url']);
    }
    if(isset($handledLeads[$post_id])) {
      return;
    }
    $job_post = get_post($post_id);
    $handledLeads[$post_id] = true;
    $attachments = array();
    $surl = get_site_url();
    $data = array();
    foreach($form['fields'] as $field) {
      $names = array($field->label, $field->adminLabel, $field->placeholder);
      $parentName = array_values(array_filter(array_map('trim', $names)))[0];
      $fields = !empty($field->inputs) ? $field->inputs : array(array('id' => $field->id, 'label' => ''));
      foreach($fields as $subField) {
        $name = $subField['label'];
        $id = $subField['id'];
        $val = rgar($lead, "$id");
        if($field->type === 'fileupload') {
          $fn = $val;
          if(strpos($fn, $surl) === 0) {
            $fn = ABSPATH.substr($fn, strlen($surl) + 1);
          }
          $attachments[] = curl_file_create($fn, '', basename($fn));
        } else {
          $data[preg_replace('/[^a-zA-Z0-9]+/', '-', $parentName.(!empty($name) ? '-'.$name:''))] = $val;
        }
      }
    }
    self::submit_apply_notification($job_post, $data, $attachments);
  }
  public static function displayApplicationForm() {
    $apply = get_option(Main::OPTION_APPPLY_FORMS);
    if(!empty($apply['gravityforms']['id'])) {
      echo gravity_form( $apply['gravityforms']['id'], empty($apply['gravityforms']['hide_title']), empty($apply['gravityforms']['hide_description']), false, null, !empty($apply['gravityforms']['use_ajax']), $tabindex, false );
    }
  }
}
GravityFormsApplications::init();


?>
