<?php
namespace com\nestdesign\WpJobManagerApi2;
class CreateTaxonomies {
  use MetaTaxonomyTrait;
  use CreateTermTrait;
  static public function init() {
    add_action('init', function () {
      self::do_create_taxonomies();
      add_action('admin_init', function () {
        self::do_create_taxonomies();
        //We need to try to install the terms to the taxonomies that we've created
        $is_imported = get_option(Main::OPTION_META_CUSTOM_TAX_IMPORTED, false);
        if(empty($is_imported)) {
          $custom_options = get_option(Main::OPTION_META_CUSTOM, array());
          if(!empty($custom_options)) {
            $did_insert = false;
            foreach($custom_options as $opt) {
              $tax = self::meta_taxonomy_name($opt);
              if(empty($tax)) continue;
              $did_insert = self::insert_default_values($tax) || $did_insert;
            }
            if($did_insert) {
              update_option(Main::OPTION_META_CUSTOM_TAX_IMPORTED, true, true);
            }
          }
        }
      }, 20, 0);
    }, 10);
  }
  static protected function do_create_taxonomies() {
    static $created = false;
    if(!empty($created))
      return false;
    $create_taxonomies = get_option(Main::OPTION_CREATE_TAXONOMY);
    if(!empty($create_taxonomies)) {
      foreach($create_taxonomies as $tax) {
        if(!empty($tax['enabled'])) {
          $created = self::do_create_taxonomy($tax, true) || $created;
        }
      }
    }
    $custom_meta = get_option(Main::OPTION_META_CUSTOM);
    if(!empty($custom_meta)) {
      foreach($custom_meta as $meta) {
        if(!empty($meta[Main::KEY_CREATE_TAXONOMY])) {
          $created = self::do_create_taxonomy($meta, false) || $created;
        }
      }
    }
    return $created;
  }
  static protected function insert_default_values($tax) {
    $did_insert = false;
    $tax_data = array(
      'job_listing_salary_currency' => array(
        array('name' => 'Euro', 'slug' => 'eur', 'meta' => array(Main::META_TAX_SYMBOL => html_entity_decode('&euro;%s'))),
        array('name' => 'British Pound', 'slug' => 'gbp', 'meta' => array(Main::META_TAX_SYMBOL => html_entity_decode('&pound;%s'))),
      ),
      'job_listing_salary_frequency' => array(
        array('name' => 'Annual', 'slug' => 'annual', 'meta' => array(Main::META_TAX_FREQUENCY => 'per anumn')),
        array('name' => 'Month', 'slug' => 'month', 'meta' => array(Main::META_TAX_FREQUENCY => 'per month')),
        array('name' => 'Week', 'slug' => 'week', 'meta' => array(Main::META_TAX_FREQUENCY => 'per week')),
        array('name' => 'Day', 'slug' => 'day', 'meta' => array(Main::META_TAX_FREQUENCY => 'per day')),
        array('name' => 'Hour', 'slug' => 'hour', 'meta' => array(Main::META_TAX_FREQUENCY => 'per hour')),
      ),
    );
    if(isset($tax_data[$tax])) {
      $did_insert = self::import_terms($tax, $tax_data[$tax]);
    }
    return $did_insert;
  }
  static protected function do_create_taxonomy($def, $is_visible = true) {
    $tax_name = isset($def[Main::KEY_TAXONOMY_NAME]) ? $def[Main::KEY_TAXONOMY_NAME] : Main::currentPostType().'_'.trim($def[Main::KEY_META_KEY], '_');
    $args = array(
      'hierarchical'      => $is_visible,
      'update_count_callback' => '_update_post_term_count',
      'label'             => $def[Main::KEY_PLURAL],
      'labels' => array(
        'name'            => $def[Main::KEY_PLURAL],
        'singular_name'   => $def[Main::KEY_SINGULAR],
        'search_items'    => sprintf( __( 'Search %s', 'wp-job-manager' ), $def[Main::KEY_PLURAL] ),
        'all_items'       => sprintf( __( 'All %s', 'wp-job-manager' ), $def[Main::KEY_PLURAL] ),
        'parent_item'     => sprintf( __( 'Parent %s', 'wp-job-manager' ), $def[Main::KEY_SINGULAR] ),
        'parent_item_colon' => sprintf( __( 'Parent %s:', 'wp-job-manager' ), $def[Main::KEY_SINGULAR] ),
        'edit_item'       => sprintf( __( 'Edit %s', 'wp-job-manager' ), $def[Main::KEY_SINGULAR] ),
        'update_item'     => sprintf( __( 'Update %s', 'wp-job-manager' ), $def[Main::KEY_SINGULAR] ),
        'add_new_item'    => sprintf( __( 'Add New %s', 'wp-job-manager' ), $def[Main::KEY_SINGULAR] ),
        'new_item_name'   => sprintf( __( 'New %s Name', 'wp-job-manager' ),  $def[Main::KEY_SINGULAR] )
      ),
      'show_ui'         => true,
      'public'          => $is_visible,
      'show_in_menu'    => $is_visible,
      'show_in_quick_edit' => $is_visible,
      'rewrite'         => false,
    );
    if(!$is_visible) {
      $args['meta_box_cb'] = false;
    }
    $o = register_taxonomy( $tax_name,
      array( Main::currentPostType() ),
      apply_filters(Main::ACTION_PREFIX.'_'.__FUNCTION__.'', $args)
    );
  }
}
CreateTaxonomies::init();
