<?php
namespace com\nestdesign\WpJobManagerApi2;

trait CreateTermTrait {
  static protected function import_terms($taxonomy, $terms_details) {
    //FIXME: We are not accounting for terms with the same slug in diferrent parents
    //We need to redo the IDs here (including parents!)
    $success = false;
    $term_parents = $existing_terms = array();
    $terms = get_terms(array('taxonomy' => $taxonomy, 'hide_empty' => false));
    foreach($terms as $t) {
      $existing_terms[$t->slug] = $t->term_id;
      $term_parents[$t->term_id] = $t->parent;
    }
    $insert_orig_new_id = array();
    $insert_parent_missing = array();
    foreach($terms_details as $term_detail) {
      if(!isset($term_detail['slug']) || !isset($existing_terms[$term_detail['slug']])) {
        //Fix the term meta
        $name = $term_detail['name'];
        //Find terms parent term
        $missing_parent = null;
        if(!empty($term_detail['parent'])) {
          //We need to find out the parent id (if already exists), or remember it for later update!
          if(isset($insert_orig_new_id[$term_detail['parent']])) {
            $parent = $insert_orig_new_id[$term_detail['parent']];
          } else {
            $missing_parent = $term_detail['parent'];
          }
          $term_detail['parent'] = $parent;
        }
        $orig_term_id = isset($term_detail['term_id']) ? $term_detail['term_id'] : null;
        unset($term_detail['term_id']);

        $inserted = self::insert_term_with_meta($term_detail, $taxonomy);
        //Remember missing parent id
        if(!is_wp_error($inserted)) {
          if(!empty($missing_parent)) {
            //Store for later update
            $insert_parent_missing[$inserted['term_id']] = $missing_parent;
          }
          if(!is_null($orig_term_id)) {
            //remember new maping
            $insert_orig_new_id[$orig_term_id] = $inserted['term_id'];
          }
          $success = true;
        } else {
          trigger_error('Failed to insert the term '.$term_detail['name'], E_USER_WARNING);
        }

      } else {
        // This term seem to exist, reuse
        $insert_orig_new_id[$term_detail['term_id']] = $existing_terms[$term_detail['slug']];
      }
    }
    if(!empty($insert_parent_missing)) {
      //TODO: Add missing parents here!
      foreach($insert_parent_missing as $term_id => $parent_id) {
        if(isset($insert_orig_new_id[$parent_id])) {
          wp_update_term($term_id, $taxonomy, array('parent' => $insert_orig_new_id[$parent_id]));
        } else {
          trigger_error('Failed to locate parent for '.$term_id, E_USER_WARNING);
        }
      }
    }
    return $success;
  }
  static protected function insert_term_with_meta($term_detail, $taxonomy) {
    $inserted = wp_insert_term($term_detail['name'], $taxonomy, $term_detail);
    if(!is_wp_error($inserted)) {
      if(!empty($term_detail['meta'])) {
        foreach($term_detail['meta'] as $key => $value) {
          foreach((array)$value as $val) {
            add_term_meta($inserted['term_id'], $key, $val);
          }
        }
      }
    }
    return $inserted;
  }

}
