<?php
namespace com\nestdesign\WpJobManagerApi2;
//TODO: Add validation for the mappings and custom meta (non taxonomy) (number, regex, nonhtml, etc.)
//TODO: Add ability to work on arbitrary content type so must implement a settings page if there is no underlaying plugin

//Not fully required
//TODO: Add ability to respond to requests without specific action (there may be action or command in the received data)
//TODO: WPML
//TODO: Add ability to disable job updates
//TODO: Add aditional apply form solutions: ninjforms - TODO, gravityforms - TODO, ...

//TEST: Add ability to track the source URL for visit (eg. indeed) using session cookie or session if available (when enabled)
//TEST: Run Apply only when the underlying plugin is active
//TEST: WPJB Facebook Application
//TEST: Process only enabled taxonomies / metas
//TEST: When the last occurence of meta key disappears from DB, still keep it in the dropdowns
//TEST: Add ability to put jobs to the user based on the contact_email field. Useful when you don't want to setup multiple credentials and you trust the sender(s)

//DONE: We need to remove the data in custom taxonomies we've created on uninstall (when said so by the settings, and add it to the backups!)
//DONE: We do not have a name and field for new File meta key (Add checkbox and another text field to File mappings)
//DONE: Add ability to import data into Custom Metadata taxonomies on install (currencies, frequencies)
//DONE: Do not run when wp-job-manager-plugin is not active
//DONE: Custom metadata without Create taxonomy needs field name for the job edit page (eg. _position)
//DONE: Apply Notification: CF7 - TESTED, WPJB Applications - TESTED
//DONE: Add ability to check for plugin updates
//DONE: On upload, tick all parent terms
//DONE: Settings Featured Image copy to (Added to settings, need to do it for process_request), add ability to create multiple image imports
//DONE: Add IP Filter option (added to settings, added to ProcessRequest)
//DONE: Some basic styling
//DONE: Save / restore settings
class Main {
  use MetaTaxonomyTrait;

  const OPTION_EXISTING_TAXONOMIES = 'wp_job_manager_api2_existing_taxonomies';
  const OPTION_CREATE_TAXONOMY = 'wp_job_manager_api2_create_taxonomies';
  const OPTION_META_MAPPING = 'wp_job_manager_api2_meta_mapping';
  const OPTION_META_CUSTOM = 'wp_job_manager_api2_meta_custom';
  const OPTION_META_CUSTOM_TAX_IMPORTED = 'wp_job_manager_api2_meta_custom_taxonomies_imported';
  const OPTION_STATUS_ON_INSERT = 'wp_job_manager_api2_status_on_insert';
  const OPTION_NEW_DATE_ON_UPDATE = 'wp_job_manager_api2_new_date_on_update';
  const OPTION_USER_USE_EMAIL = 'wp_job_manager_api2_attach_email_based';
  const OPTION_ENABLE_APPLICATION_NOTIFICATION = 'wp_job_manager_api2_enable_application_notification';
  const OPTION_ENABLE_APPLICATION_SOURCE_TRACK = 'wp_job_manager_api2_enable_application_source_track';
  const OPTION_ENABLE_APPLICATION_IN_JSON = 'wp_job_manager_api2_enable_application_in_json';
  const OPTION_IP_FILTER = 'wp_job_manager_api2_ip_filter';
  const OPTION_JOB_URL = 'wp_job_manager_api2_job_url';
  const OPTION_PLUGIN_CLEANUP = 'wp_job_manager_api2_on_delete_cleanup';
  const OPTION_LOG_REQUESTS = 'wp_job_manager_api2_log_requests';
  const OPTION_FILES = 'wp_job_manager_api2_files';
  const OPTION_APPPLY_FORMS = 'wp_job_manager_api2_apply_forms';

  const KEY_CREATE_TAXONOMY = 'create_taxonomy';
  const KEY_TAXONOMY_NAME = 'taxonomy_name';
  const KEY_SINGULAR = 'singular';
  const KEY_PLURAL = 'plural';
  const KEY_ENABLED = 'enabled';
  const KEY_VALID_REQUIRED = 'valid_required';
  const KEY_DEFAULT_ENABLED = 'default_enabled';
  const KEY_FILE_CUSTOM_ID_FIELD = 'id_custom';
  const KEY_FILE_CUSTOM_URL_FIELD = 'url_custom';
  const KEY_DEFAULT_VALUE = 'default_value';
  const KEY_IS_ATTACHMENT = 'is_attachment';
  const KEY_ATTACHMENT_ID_FIELDS = 'id';
  const KEY_ATTACHMENT_URL_FIELDS = 'url';
  const KEY_APPLICATION_SOURCE_TRACK = 'application_source_tracking';
  const KEY_ITEM_ENABLED = 'enabled';
  const KEY_SEPARATOR_ENABLE = 'separator_enabled';
  const KEY_SEPARATOR = 'separator';
  const KEY_TAXONOMY_SEARCH_NAME = 'search_by_name';


  const KEY_META_KEY = 'meta_key';
  const KEY_META_TITLE = 'meta_title';
  const KEY_FIELDS = 'fields';
  const KEY_ENABLE_CREATE_TERM = 'enable_create';
  const KEY_ENABLE_REPORTS = 'enable_reports';

  const VALUE_LOG_REQUESTS_FAILED = 'failed';
  const VALUE_LOG_REQUESTS_ALL = 'both';
  const VALUE_LOG_REQUESTS_NONE = 'none';

  const ADMINAJAX_ACTION_PREFIX = 'wpjb_api';
  const ACTION_PREFIX = 'wpjb_api';
  const ACTION_DISPLAY_APPLY_FORM = 'wpjb_api2_display_application';

  const INPUT_LOGIN_USERNAME = 'login_username';
  const INPUT_LOGIN_PASSWORD = 'login_password';

  const META_APPLICATION = '_application'; //This depends on WP Job Manager default field name
  const META_APPLICATION_URL = '_apply_notify_url';
  const META_ATTACHMENT_IMPORT = '_thumbnail_id';
  const META_ATTACHMENT_DOWNLOADED_FROM = '_wpjb_api_downloaded_from';
  const META_REMOTE_ID = '_remote_id';

  const META_TAX_SYMBOL = '_symbol';
  const META_TAX_FREQUENCY = '_salary_string';

  //Might be fun to do this for JobBoard aswell!
  static public function currentPostType() {
    include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    if(is_plugin_active( 'wp-job-manager/wp-job-manager.php' )) {
      return 'job_listing';
    } else if (is_plugin_active( 'simple-job-board/simple-job-board.php' )) {
      return 'jobpost';
    } else if (post_type_exists( 'noo_job' )) {
      return 'noo_job';
    }
  }
  static public function applicationEmailField() {
    if (self::isNooJob()) {
      return '_application_email';
    }
    return Main::META_APPLICATION;
  }
  static public function isWPJM() {
    include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    return is_plugin_active( 'wp-job-manager/wp-job-manager.php' );
  }
  static public function isSimpleJobBoard() {
    include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    return is_plugin_active('simple-job-board/simple-job-board.php');
  }
  static public function isNooJob() {
    return post_type_exists( 'noo_job' );
  }
  static public function init() {
    if(!self::isWPJM() && !self::isSimpleJobBoard() && !self::isNooJob()) {
      add_action('admin_notices', function () {
        echo '<div class="notice notice-error"><p>'.sprintf(htmlspecialchars(__('WP Job Manager API2 requires %1$sWP Job Manager plugin%2$s or %3$sSimple Job Board%4$s to be activated or that \'noo_job\' post type is available.', 'wp-job-manager-api2'), ENT_QUOTES), '<a href="https://cs.wordpress.org/plugins/wp-job-manager/" target="_blank">', '</a>', '<a href="https://cs.wordpress.org/plugins/simple-job-board/">', '</a>').'</p></div>';
      });
      return;
    }
    require_once(dirname(__FILE__).'/create_taxonomies.php');
    require_once(dirname(__FILE__).'/settings.php');

    $check_request = false;
    if(defined('DOING_AJAX') && DOING_AJAX) {
      $check_request = 'ajax';
    } //We may define more endpoints, eg. WP Rest API?
    if($check_request !== false) {
      require_once(dirname(__FILE__).'/process_request.php');
      if($check_request === 'ajax') {
        require_once(dirname(__FILE__).'/admin_ajax.php');
      }
    }
    require_once(dirname(__FILE__).'/post_meta.php');
    require_once(dirname(__FILE__).'/term_meta.php');
    if(get_option(Main::OPTION_ENABLE_APPLICATION_NOTIFICATION, false)) {
      require_once(dirname(__FILE__).'/applications/applications.php');
    }
    self::registerPluginActions();
    add_action( 'init', 'com\nestdesign\WpJobManagerApi2\Main::load_plugin_textdomain' , 12 );
    add_action( 'init', 'com\nestdesign\WpJobManagerApi2\Main::action_init' , 12 );
  }
  static protected function registerPluginActions() {
    register_activation_hook( dirname(dirname(__FILE__)).'/wp-job-manager-api2.php', 'com\nestdesign\WpJobManagerApi2\Main::install');
    register_deactivation_hook( dirname(dirname(__FILE__)).'/wp-job-manager-api2.php', 'com\nestdesign\WpJobManagerApi2\Main::uninstall');
  }
  static public function install() {
    //We should try to add the initial config data (create terms in taxonomies, ...)
  }
  static public function uninstall() {
    //Depending on the setting Main::OPTION_PLUGIN_CLEANUP, we should delete our existing settings
    $remove = get_option(Main::OPTION_PLUGIN_CLEANUP, false);
    if(!empty($remove)) {
      $custom_tax = get_option(Main::OPTION_CREATE_TAXONOMY, array());
      if(!empty($custom_tax)) {
        foreach($custom_tax as $def) {
          if(!empty($def[Main::KEY_TAXONOMY_NAME])) {
            self::delete_all_terms($def[Main::KEY_TAXONOMY_NAME]);
          }
        }
      }
      $custom_meta = get_option(Main::OPTION_META_CUSTOM, array());
      $deleted = false;
      if(!empty($custom_meta)) {
        foreach($custom_meta as $def) {
          $tax = self::meta_taxonomy_name($def);
          if(!empty($tax)) {
            $deleted = self::delete_all_terms($tax) || $deleted;
          }
        }
      } else {
        die('No Custom Meta taxonomies?');
      }
      foreach(self::getAllOptionConstants() as $const) {
        delete_option(constant('com\nestdesign\WpJobManagerApi2\Main::'. $const));
      }
    }
  }
  static protected function delete_all_terms($taxonomy) {
    $terms = get_terms(array('taxonomy' => $taxonomy, 'hide_empty' => false));
    if(empty($terms)) {
      return false;
    }
    foreach($terms as $term) {
      $res = wp_delete_term($term->term_id, $taxonomy);
      if(is_wp_error($res)) {
        var_dump($res);
        die('Failed to delete term');
        return false;
      }
    }
    return true;
  }
  static public function getAllOptionConstants() {
    $rc = (new \ReflectionClass(get_class()));
    $constants = is_callable(array($rc, 'getReflectionConstants')) ? array_map(function ($o) { return $o -> name; }, $rc->getReflectionConstants()) : array_keys($rc->getConstants());
    return array_filter($constants, function ($const) {
      return strpos($const, 'OPTION_') === 0;
    });
  }
  static public function action_init() {
    $tracking_source_enabled = get_option(Main::OPTION_ENABLE_APPLICATION_SOURCE_TRACK, false);
    if(!empty($tracking_source_enabled)) {
      $application_source_to_track = null;
      if(isset($_SERVER['HTTP_REFERER'])) {
        $host = parse_url($_SERVER['HTTP_REFERER'], PHP_URL_HOST);
        if(strpos($host, $_SERVER['HTTP_HOST']) === false) {
          $application_source_to_track = $_SERVER['HTTP_REFERER'];
        }
      }
      if(!empty($application_source_to_track)) {
        //so now lets setup the storage (session or cookie?)
        if(!empty(session_id())) {
          //we have session, hope we will have it when we reach the application submission!
          $_SESSION[Main::KEY_APPLICATION_SOURCE_TRACK] = $application_source_to_track;
        } else {
          //use cookie
          setCookie(Main::KEY_APPLICATION_SOURCE_TRACK, $application_source_to_track, 0, '/');
        }
      }
    }
  }
  static public function load_plugin_textdomain() {
    $locale = apply_filters( 'plugin_locale', get_locale(), 'wp-job-manager-api2' );
    $locale = 'cz_CZ';
    load_textdomain( 'wp-job-manager-api2', WP_LANG_DIR . "/wp-job-manager-api2/wp-job-manager-api2-$locale.mo" );
    load_plugin_textdomain( 'wp-job-manager-api2', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
  }
}
add_action('init', function () {
  Main::init();
}, 1);
