<?php
namespace com\nestdesign\WpJobManagerApi2;

Trait OutputTrait {
  static protected function is_json($mime) {
    return in_array(strToLower($mime), array('application/json', 'text/json'));
  }
  static protected function is_xml($mime) {
    return in_array(strToLower($mime), array('application/xml', 'text/xml'));
  }
  static public function output_result($result) {
    $mime = reset(explode(',', $_SERVER['HTTP_ACCEPT']));
    $result = self::update_response_status($result);
    if((empty($_REQUEST['format']) && self::is_json($mime)) || $_REQUEST['format'] === 'json') {
      Header('Content-Type: application/json');
      self::encode_response(json_encode($result, JSON_PRETTY_PRINT)."\n");
    } else if ((empty($_REQUEST['format']) && self::is_xml($mime)) || $_REQUEST['format'] === 'xml') {
      Header('Content-Type: application/xml');
      self::encode_response("<xmlfeed>\n".self::array_to_xml($result)."\n</xmlfeed>\n");
    }
    self::encode_response("Submit correct Accept header, or force by format=json/format=xml parameter\n<pre>\n".print_r($result, true).'</pre>'."\n");
  }
  static protected function update_response_status($result) {
    $status_code = 200;
    if(isset($result['code'])) {
      $status_code = (int)$result['code'];
      unset($result['code']);
    }
    if(isset($result['status'])) {
      if($status_code === 200 && $result['status'] !== 'success') {
        $status_code = 500;
      }
      unset($result['status']);
    }
    if($status_code !== http_response_code()) {
      http_response_code($status_code);
    }
    return $result;
  }
  static public function encode_response($data) {
    if(!headers_sent()) {
      $debug = '';
      if(ob_get_level() > 0) {
        while (ob_get_level() > 0) {
          $debug .= ob_get_clean();
        }
        $data = $debug.$data;
      }
      if(strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') !== false && function_exists('ob_gzhandler')) {
        ob_start('ob_gzhandler');
      }
    } else {
      echo "Headers were sent\n";
    }
    die($data);
  }
  /**
  * array_to_xml() - transfer array to XML string
  * @param type $item - array that should be transferred to XML
  * @param type $xml_settings - array of settings eg: array('checkpoints' => array('key'=>'id', 'array_item_name'=>'checkpoint'));
  * @param type $parentkey - usually used only for recursive call.
  * @return string XML
  * @from challenger
  */
  static private function array_to_xml($item, $xml_settings=array(), $parentkey=''){
    $str ='';
    foreach($item as $key=>$value) {
      $add_attr = isset($xml_settings[$parentkey]['key']) ? ' '.$xml_settings[$parentkey]['key'].'="'.htmlspecialchars($key).'"' : '';
      if(!empty($xml_settings[$parentkey])){
        foreach($xml_settings[$parentkey] as $k => $v){
          if($k!='key' AND $k != 'array_item_name' && $v === true){
            $add_attr .= ' '.$k.'="'.htmlspecialchars($value[$k]).'"';
            if(is_array($value))
              unset($value[$k]);
          }
        }
      }
      if(!empty($xml_settings[$key])){
        foreach($xml_settings[$key] as $k => $v){
          if($k!='key' AND $k != 'array_item_name' && is_string($v)){
            $add_attr .= ' '.$k.'="'.htmlspecialchars($v).'"';
          }
        }
      }
      $out_value = null;
      if(is_array($value)) {
        $out_value = self::array_to_xml($value, $xml_settings, $key);
      } else if (empty($xml_settings['use_cdata']) || is_int($value) || is_numeric($value)) {
        if(!empty($xml_settings['latin_to_utf']) && is_string($value))
          $value = iconv("cp1252","UTF-8",$value);
        if(!empty($xml_settings['htmlspecialchars_encoding'])) {
          $out_value = htmlspecialchars($value, 0, $xml_settings['htmlspecialchars_encoding']);
        } else {
          $out_value = htmlspecialchars($value);
        }
      } else {
        if(!empty($xml_settings['latin_to_utf']))
          $value = iconv("cp1252","UTF-8",$value);
        $out_value = '<![CDATA['.$value.']]>';
      }
      if(!is_int($key) && (!empty($value) || $value === '0')){
        $array_item_name2 = isset($xml_settings[$parentkey]['array_item_name']) ? $xml_settings[$parentkey]['array_item_name'] : $key;
        $str .= '<'.htmlspecialchars($array_item_name2).$add_attr.'>';
        $str .= $out_value;
        $str .= '</'.htmlspecialchars($array_item_name2).'>'."\n";
      }elseif(is_int($key)){
        $array_item_name2 = isset($xml_settings[$parentkey]['array_item_name']) ? $xml_settings[$parentkey]['array_item_name'] : 'item';
        $str .= '<'.$array_item_name2.$add_attr.'>'.($out_value).'</'.$array_item_name2.'>'."\n";
      }
    }
    return $str;
  }
}
