<?php
namespace com\nestdesign\WpJobManagerApi2;


class PostMeta {
  use MetaTaxonomyTrait;
  protected static $checkboxes = array('_salary_negotiable');
  static public function init() {
    self::registerWPJBFilters();
    if(Main::isSimpleJobBoard()) {
      self::registerSimpleJobBoardMetaBoxes();
    }
  }
  static protected function registerSimpleJobBoardMetaBoxes() {
    add_action('save_post_'.Main::currentPostType(), function ($post_id) {
      // Check if nonce is set.
      if ( NULL == filter_input( INPUT_POST, 'nest_jobpost_meta_box_nonce' ) ) {
        return;
      }
      // Verify that the nonce is valid.
      check_admin_referer( 'sjb_nest_jobpost_meta_box', 'nest_jobpost_meta_box_nonce' );

      // If this is an autosave, our form has not been submitted, so we don't want to do anything.
      if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
      }

      // Check the user's permissions.
      if ( NULL != filter_input( INPUT_POST, 'post_type' ) && 'page' == filter_input( INPUT_POST, 'post_type') ) {
        if (!current_user_can('edit_page', $post_id)) {
          return;
        }
      } else {
        if (!current_user_can('edit_post', $post_id)) {
          return;
        }
      }
      // Now save the data!
      $fields = apply_filters('job_manager_job_listing_data_fields', array());
      if(!empty($fields)) {
        $POST_data = filter_input_array( INPUT_POST );
        foreach($fields as $key => $item) {
          if(isset($POST_data['nest_sjb'][$key])) {
            update_post_meta($post_id, $key, $POST_data['nest_sjb'][$key]);
          } else {
            delete_post_meta($post_id, $key);
          }
        }

      }
    });
    add_action( 'add_meta_boxes', function () {
      add_meta_box('jobpost_metas', sprintf(esc_html__('%s API Fields', 'simple-job-board'), $wp_post_types['jobpost']->labels->singular_name), function ($post) {
        $fields = apply_filters('job_manager_job_listing_data_fields', array());
        if(!empty($fields)) {
          wp_nonce_field('sjb_nest_jobpost_meta_box', 'nest_jobpost_meta_box_nonce');

          echo '<div class="job_features meta_option_panel jobpost_fields">';
          echo '  <ul id="job_features">';
          $meta = get_post_meta($post->ID);
          foreach($fields as $key => $item) {
            $label = $item['label'];
            $value = $meta[$key][0];

            echo '<li><label class="sjb-label" for="meta_nest_'.$key.'">'. $label . '</label><input type="text" id="meta_nest_'.$key.'" name="nest_sjb[' . $key . ']" value="' . $value . '" /></li>';
          }
          echo '  </ul>';
          echo '</div>';

        }
      }, Main::currentPostType(), 'normal', 'high');
    } );
  }
  static protected function registerWPJBFilters() {
    add_filter('job_manager_job_listing_data_fields', function ($fields) {
      $fields[Main::META_REMOTE_ID] = array(
        'label' => __( 'Remote ID', 'wp-job-manager-api2' ),
        'placeholder' => __( 'This is the ID the poster assigned to the job', 'wp-job-manager-api2' ),
        'description' => __( 'This is the unique identifier for the job, useful when posting using the API', 'wp-job-manager-api2' ),
      );
      if(!empty(get_option(Main::OPTION_ENABLE_APPLICATION_NOTIFICATION))) {
        $fields[Main::META_APPLICATION_URL] = array(
          'label' => __( 'Apply Notification URL', 'wp-job-manager-api2' ),
          'placeholder' => __( 'URL where a notification about an application is to be sent', 'wp-job-manager-api2' ),
          'description' => __( 'This URL will receive the entered data and the attachment (if present)', 'wp-job-manager-api2' ),
        );
      }
      $meta_custom = get_option(Main::OPTION_META_CUSTOM, array());
      if(!empty($meta_custom)) {
        foreach($meta_custom as $def) {
          if(!empty($def[Main::KEY_CREATE_TAXONOMY]) && isset($def[Main::KEY_SINGULAR])) {
            if(!empty($def[Main::KEY_SINGULAR])) {
              $options = array();
              foreach(get_categories(array('taxonomy' => self::meta_taxonomy_name($def), 'hide_empty' => false)) as $cat) {
                $options[$cat -> slug] = $cat -> name;
              }
              $fields[$def[Main::KEY_META_KEY]] = array(
                'label' => $def[Main::KEY_SINGULAR],
                'type' => 'select',
                'options' => $options,
              );
            }
          } else {
            if(!empty($def[Main::KEY_META_KEY])) {
              $fields[$def[Main::KEY_META_KEY]] = array(
                'label' => !empty($def[Main::KEY_META_TITLE]) ? $def[Main::KEY_META_TITLE] : ucfirst(trim(str_replace('_', ' ', $def[Main::KEY_META_KEY]))),
                'type' => 'text',
              );
            }
          }
        }
      }
      $meta_mapping = get_option(Main::OPTION_META_MAPPING, array());
      if(!empty($meta_mapping)) {
        foreach($meta_mapping as $def) {
          if(!empty($def[Main::KEY_META_KEY]) && !in_array($def[Main::KEY_META_KEY], Settings::$main_post_meta, true) && !isset($fields[$def[Main::KEY_META_KEY]])) {
            $fields[$def[Main::KEY_META_KEY]] = array(
              'label' => !empty($def[Main::KEY_META_TITLE]) ? $def[Main::KEY_META_TITLE] : ucfirst(trim(str_replace('_', ' ', $def[Main::KEY_META_KEY]))),
              'type' => in_array($def[Main::KEY_META_KEY], self::$checkboxes, true) ? 'checkbox' : 'text',
            );
          }
        }
      }
      $files = get_option(Main::OPTION_FILES, array());
      if(!empty($files)) {
        foreach($files as $def) {
          if(!empty($def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_ITEM_ENABLED]) && !empty($def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_META_KEY])) {
            $fields[$def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_META_KEY]] = array(
              'label' => !empty($def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_SINGULAR]) ? $def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_SINGULAR] : ucfirst(trim(str_replace('_', ' ', $def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_META_KEY]))),
              'type' => in_array($def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_META_KEY], self::$checkboxes, true) ? 'checkbox' : 'text',
            );
          }
          if(!empty($def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_ITEM_ENABLED]) && !empty($def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_META_KEY])) {
            $fields[$def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_META_KEY]] = array(
              'label' => !empty($def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_SINGULAR]) ? $def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_SINGULAR] : ucfirst(trim(str_replace('_', ' ', $def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_META_KEY]))),
              'type' => in_array($def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_META_KEY], self::$checkboxes, true) ? 'checkbox' : 'text',
              'std' => '',
            );
          }
        }
      }
      return $fields;
    }, 100, 1);
  }
}
PostMeta::init();
