<?php
namespace com\nestdesign\WpJobManagerApi2;
require_once(dirname(__FILE__).'/errors/wp_error_exception.php');
//TODO: We may want to add some more details to active reports based on the settings
class ProcessRequest {
  use MetaTaxonomyTrait;
  private static $is_checking_mapping = false;
  protected static $warnings = array();
  static public function init() {
    self::registerActions();
  }
  static protected function registerActions() {
    $log_requests = get_option(Main::OPTION_LOG_REQUESTS);
    foreach(self::endpoints() as $fn_name) {
      add_filter(Main::ACTION_PREFIX.'_'.$fn_name.'_init', 'com\nestdesign\WpJobManagerApi2\ProcessRequest::checkIpFilter', 10, 2);
      if($log_requests != Main::VALUE_LOG_REQUESTS_NONE) {
        if(in_array($log_requests, array(Main::VALUE_LOG_REQUESTS_FAILED, Main::VALUE_LOG_REQUESTS_ALL), true)) {
          add_filter(Main::ACTION_PREFIX.'_'.$fn_name.'_error', 'com\nestdesign\WpJobManagerApi2\ProcessRequest::log_request', 10, 2);
        }
        if($log_requests === Main::VALUE_LOG_REQUESTS_ALL) {
          add_filter(Main::ACTION_PREFIX.'_'.$fn_name.'_success', 'com\nestdesign\WpJobManagerApi2\ProcessRequest::log_request', 10, 2);
        }
      }
    }
    $allow_option_user_use_email = get_option(Main::OPTION_USER_USE_EMAIL, false);
    if(!empty($allow_option_user_use_email)) {
      add_filter(Main::ACTION_PREFIX.'_author_id', 'com\nestdesign\WpJobManagerApi2\ProcessRequest::author_for_data', 10, 3);
    }
  }
  static public function author_for_data($author_id, $data) {
    //Find mapping!
    $meta_mapping = get_option(Main::OPTION_META_MAPPING);
    $meta_types = array_column($meta_mapping, null, Main::KEY_META_KEY);
    if(isset($meta_types[Main::applicationEmailField()])) {
      foreach($meta_types[Main::applicationEmailField()][Main::KEY_FIELDS] as $field) {
        if(empty($field)) continue;
        if(isset($data[$field]) && strpos($data[$field], '@') !== false) {
          $user = get_user_by('email', $data[$field]);
          if(!is_wp_error($user) && !empty($user->ID)) {
            return $user->ID;
          }
        }
      }
    }
    return $author_id;
  }
  static public function log_request($output, $filter) {
    $upl = wp_upload_dir();
    $fn = $upl['basedir'].'/wp-job-manager-api2-'.gmdate('Y-m-d').'.log';
    $request = $_REQUEST;
    $server = $_SERVER;
    foreach($filter as $key => $itm) {
      if(is_object($itm) && isset($itm -> ID)) {
        $filter[$key] = get_class($itm).'::'.$itm -> ID;
      }
    }
    file_put_contents($fn, "\n--------------------------\n".gmdate('Y-m-d H:i:s')." GMT\n".preg_replace('/^(\s*\'.*(pass|author).*\' => )\'.*\',$/mi', '\1\'***\',', var_export(compact('output', 'filter', 'request', 'server'), true)), FILE_APPEND);
    return $output;
  }
  static private function cidr_match($ip, $range) {
    list ($subnet, $bits) = explode('/', $range);
    $ip = ip2long($ip);
    $subnet = ip2long($subnet);
    $mask = -1 << (32 - $bits);
    $subnet &= $mask; # nb: in case the supplied subnet wasn't correctly aligned
    return ($ip & $mask) == $subnet;
  }
  static public function checkIpFilter($res, $data) {
    $filter = get_option(Main::OPTION_IP_FILTER);
    if(!empty($filter[Main::KEY_ENABLED]) && !empty($filter['ips'])) {
      $valid = false;
      foreach($filter['ips'] as $ip) {
        if(empty($ip)) {
          continue;
        }
        if(strpos($ip, '/') !== false) {
          if(self::cidr_match($_SERVER['REMOTE_ADDR'], $ip)) {
            $valid = true;
          }
        } else if($_SERVER['REMOTE_ADDR'] === $ip) {
          $valid = true;
          break;
        }
      }
      if(!$valid) {
        return ProcessRequest::result_error(array('ip_filter_failed'), __FUNCTION__, $data, 403);
      }
    }
    return $res;
  }
  // API functions
  static public function API_upload_job($data) {
    $login = self::check_login($data);
    if(is_wp_error($login) || $login === false) {
      return ProcessRequest::result_error(array(($login === false ? 'missing' : 'invalid') => 'login_details'), __FUNCTION__, compact('data', 'login'), 403);
    }
    $result = apply_filters(Main::ACTION_PREFIX.'_'.__FUNCTION__.'_init', null, compact('data', 'login'));
    if(!is_null($result)) {
      return $result;
    }

    $errors = self::has_all_required_fields($data);
    if(!empty($errors)) {
      return ProcessRequest::result_error($errors, __FUNCTION__, compact('data', 'login'), 400);
    }
    $remote_id = null;
    try {
      $post_data = self::do_mapping($data, $remote_id);
    } catch (\com\nestdesign\WpJobManagerApi2\Exceptions\WpErrorException $ex) {
      return ProcessRequest::result_error(array($ex->getMessage()), __FUNCTION__, array_merge(compact('data', 'post_data', 'remote_id', 'login'), array('wp_error' => $ex->wpError)));
    } catch(\Exception $ex) {
      return ProcessRequest::result_error(array($ex->getMessage()), __FUNCTION__, compact('data', 'post_data', 'remote_id', 'login'));
    }
    if(empty(trim($remote_id))) {
      return ProcessRequest::result_error(array('remote_id not supplied'), __FUNCTION__, compact('data', 'post_data', 'remote_id', 'login'), 400);
    }
    $post_data['ID'] = self::find_job_by_remoteid($remote_id, $login, $data);
    if(empty($post_data['ID'])) {
      unset($post_data['ID']);
      $post_data['post_status'] = get_option(Main::OPTION_STATUS_ON_INSERT);
    } else {
      //We need to do this as wp_insert_post would change that to draft
      $post_data['post_status'] = self::get_post_status($post_data['ID']);
    }
    $job_url = get_option(Main::OPTION_JOB_URL, false);
    if(!empty($job_url[Main::KEY_ENABLED])) {
      $post_data = self::update_job_url($post_data, $job_url);
    }
    if(empty($post_data['ID']) || !empty(get_option(Main::OPTION_NEW_DATE_ON_UPDATE, false))) {
      $post_data['post_date'] = date('Y-m-d H:i:s');
      $post_data['post_date_gmt'] = gmdate('Y-m-d H:i:s');
    }
    $post_data['post_type'] = Main::currentPostType();
    $post_data['post_author'] = apply_filters(Main::ACTION_PREFIX.'_author_id', $login->ID, $data);
    $post_ID = self::wp_insert_post($post_data);
    if(empty($post_ID)) {
      return ProcessRequest::result_error(array('Failed to insert the post'), __FUNCTION__, compact('data', 'post_data', 'login'), 500);
    } else if(is_wp_error($post_ID)) {
      return ProcessRequest::result_error($post_ID, __FUNCTION__, compact('data', 'post_data', 'login'), 500);
    }
    return ProcessRequest::result_success(array('local_id' => $post_ID, 'url' => get_permalink($post_ID), 'message' => 'Successfuly posted'), __FUNCTION__, compact('data', 'post_data', 'login'));
  }
  static protected function update_job_url($post_data, $job_url) {
    if(!empty($job_url['only_new']) && !empty($post_data['ID'])) {
      return $post_data;
    }
    $fields = array_filter($job_url['fields']);
    if(empty($fields)) {
      return $post_data;
    }
    $taxes = get_taxonomies();
    $url_parts = array();
    foreach($fields as $key) {
      $part = isset($post_data[$key]) ? $post_data[$key] : (isset($post_data['meta_input'][$key]) ? $post_data['meta_input'][$key] : null);
      if(is_null($part) && isset($taxes[$key])) {
        if(isset($post_data['tax_input'][$key])) {
          $terms = get_terms(array('taxonomy' => $key, 'include' => $post_data['tax_input'][$key]));
          if(!empty($terms)) {
            $part = implode('-', array_map(function ($itm) { return $itm -> name; }, $terms));
          } else {
            trigger_error('Invalid terms supplied for URL: '.var_export($terms, true), E_USER_WARNING);
          }
        }
      }
      if(!is_null($part)) {
        $url_parts[] = $part;
      }
    }
    $separator = !empty($job_url['separator']) ? $job_url['separator'] : '-';
    $post_data['post_name'] = sanitize_title(implode($separator, $url_parts));
    return $post_data;
  }
  static public function API_delete_job($data) {
    $login = self::check_login($data);
    if(is_wp_error($login) || $login === false) {
      return ProcessRequest::result_error(array(($login === false ? 'missing' : 'invalid') => 'login_details'), __FUNCTION__, compact('data', 'login'), 403);
    }
    $result = apply_filters(Main::ACTION_PREFIX.'_'.__FUNCTION__.'_init', null, compact('data', 'login'));
    if(!is_null($result)) {
      return $result;
    }

    $meta_mapping = get_option(Main::OPTION_META_MAPPING, array());
    $remote_id = null;
    if(!empty($meta_mapping)) {
      foreach($meta_mapping as $def) {
        if($def[Main::KEY_META_KEY] === Main::META_REMOTE_ID) {
          $remote_id = reset(self::has_some_value($data, $def));
        }
      }
    }
    $post_ID = self::find_job_by_remoteid($remote_id, $login, $data);
    if (empty($post_ID)) {
      return ProcessRequest::result_error(array('no_items_found' => $remote_id), __FUNCTION__, compact('data', 'the_query', 'remote_id', 'post_ID', 'login'), 400);
    }
    do_action(Main::ACTION_PREFIX.'_'.__FUNCTION__.'_before_delete', $post_ID, $remote_id, $data);
    $o = self::wp_delete_post($post_ID, true);
    return ProcessRequest::result_success(array('message' => 'Successfuly deleted'), __FUNCTION__, compact('data', 'post_ID', 'remote_id', 'login'));
  }
  static public function API_active_reports($data) {
    $login = self::check_login($data);
    if(is_wp_error($login) || $login === false) {
      return ProcessRequest::result_error(array(($login === false ? 'missing' : 'invalid') => 'login_details'), __FUNCTION__, compact('data', 'login'), 403);
    }
    $result = apply_filters(Main::ACTION_PREFIX.'_'.__FUNCTION__.'_init', null, compact('data', 'login'));
    if(!is_null($result)) {
      return $result;
    }

    $query = new \WP_Query(self::search_post_arguments($login));
    if(!$query->have_posts()) {
      ProcessRequest::result_success(array(), __FUNCTION__, compact('data', 'query', 'login'));
    }
    $out = array();
    do {
      $query->the_post();
      $out[] = self::report_show_item($query->post);
    } while ($query->have_posts());
    return ProcessRequest::result_success($out, __FUNCTION__, compact('data', 'query', 'login'));
  }
  static public function API_list_taxonomy($data) {
    $login = self::check_login($data);
    if(is_wp_error($login) || $login === false) {
      return ProcessRequest::result_error(array(($login === false ? 'missing' : 'invalid') => 'login_details'), __FUNCTION__, compact('data', 'login'), 403);
    }
    $result = apply_filters(Main::ACTION_PREFIX.'_'.__FUNCTION__.'_init', null, compact('data', 'login'));
    if(!is_null($result)) {
      return $result;
    }

    $available_taxonomies = self::available_taxonomies();
    if(empty($data['taxonomy'])) {
      return ProcessRequest::result_error(array('missing' => array('taxonomy'), 'taxonomy_choices' => array_keys($available_taxonomies)), __FUNCTION__, compact('data', 'login'), 400);
    } else if (!isset($available_taxonomies[$data['taxonomy']])) {
      return ProcessRequest::result_error(array('invalid_taxonomy' => $data['taxonomy'], 'taxonomy_choices' => array_keys($available_taxonomies)), __FUNCTION__, compact('data', 'login'), 404);
    }

    $terms = get_terms(array('taxonomy' => $data['taxonomy'], 'hide_empty' => false));
    if(is_wp_error($terms)) {
      ProcessRequest::result_error($terms, __FUNCTION__, compact('data', 'login'), 500);
    }
    return ProcessRequest::result_success(array_map('self::list_show_term', $terms, array_fill(0, count($terms), $available_taxonomies[$data['taxonomy']])), __FUNCTION__, compact('data', 'login'));
  }
  static protected function available_taxonomies () {
    $available_taxonomies = array();
    $options = array(Main::OPTION_META_CUSTOM, Main::OPTION_EXISTING_TAXONOMIES, Main::OPTION_CREATE_TAXONOMY);
    foreach($options as $option) {
      $values = get_option($option, array());
      foreach($values as $def) {
        if(!empty($def[Main::KEY_TAXONOMY_NAME])) {
          $available_taxonomies[$def[Main::KEY_TAXONOMY_NAME]] = true;
        } else if (!empty($def[Main::KEY_CREATE_TAXONOMY]) && !empty($def[Main::KEY_META_KEY])) {
          $available_taxonomies[self::meta_taxonomy_name($def)] = false;
        }
      }
    }
    return $available_taxonomies;
  }

  // Mapping functions
  static protected function do_mapping($data, &$remote_id = null) {
    $post_data = array();

    //Ok We need to load all the settings and check what goes where!
    self::do_mapping_meta($data, Main::OPTION_META_MAPPING, $post_data, $remote_id);
    self::do_mapping_meta($data, Main::OPTION_META_CUSTOM, $post_data);
    self::do_mapping_check_image($data, Main::OPTION_FILES, $post_data);
    self::do_mapping_taxonomy($data, Main::OPTION_META_CUSTOM, $post_data);
    self::do_mapping_taxonomy($data, Main::OPTION_EXISTING_TAXONOMIES, $post_data);
    self::do_mapping_taxonomy($data, Main::OPTION_CREATE_TAXONOMY, $post_data);

    return apply_filters(Main::ACTION_PREFIX.'_'.$function, $post_data, $data, $remote_id);
  }
  static protected function do_mapping_meta($data, $option, &$post_data, &$remote_id = null) {
    $meta_mapping = get_option($option, array());
    foreach($meta_mapping as $def) {
      if(empty($def[Main::KEY_META_KEY])) continue;
      if(!empty($def[Main::KEY_CREATE_TAXONOMY])) continue;
      $values = self::has_some_value($data, $def);
      if($values) {
        if(in_array($def[Main::KEY_META_KEY], Settings::$main_post_meta, true)) {
          if($def[Main::KEY_META_KEY] === 'post_status' && !in_array(reset($values), get_post_stati(), true)) continue;
          $post_data[$def[Main::KEY_META_KEY]] = reset($values);
        } else {
          $val = reset($values);
          $post_data['meta_input'][$def[Main::KEY_META_KEY]] = $val;
          if($def[Main::KEY_META_KEY] === Main::META_REMOTE_ID) {
            $remote_id = $val;
          }
        }
      } else {
        if(in_array($def[Main::KEY_META_KEY], Settings::$main_post_meta, true)) {
          $post_data[$def[Main::KEY_META_KEY]] = '';
        } else {
          $post_data['meta_input'][$def[Main::KEY_META_KEY]] = '';
        }
      }
    }
  }
  static protected function do_mapping_check_image($data, $option, &$post_data) {
    $meta_mapping = get_option($option, array());
    foreach($meta_mapping as $def) {
      if(empty($def[Main::KEY_FIELDS])) continue;
      if(empty($def[Main::KEY_ATTACHMENT_ID_FIELDS]) && empty($def[Main::KEY_ATTACHMENT_URL_FIELDS])) continue;
      $values = self::has_some_value($data, $def);
      if($values) {
        $val = reset($values);
        //We need to download the file
        list($img_id, $img_url) = self::image_from_url($val);
        if(empty($img_id)) {
          $img_id = '';
          $img_url = '';
        }
        if(!empty($def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_ITEM_ENABLED]) && !empty($def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_META_KEY])) {
          $post_data['meta_input'][$def[Main::KEY_FILE_CUSTOM_ID_FIELD][Main::KEY_META_KEY]] = $img_id;
        }
        if(!empty($def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_ITEM_ENABLED]) && !empty($def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_META_KEY])) {
          $post_data['meta_input'][$def[Main::KEY_FILE_CUSTOM_URL_FIELD][Main::KEY_META_KEY]] = $img_url;
        }
        //Attach the values to the meta items
        foreach((array)$def[Main::KEY_ATTACHMENT_ID_FIELDS] as $idField) {
          if(!empty($idField)) {
            $post_data['meta_input'][$idField] = $img_id;
          }
        }
        foreach((array)$def[Main::KEY_ATTACHMENT_URL_FIELDS] as $urlField) {
          if(!empty($urlField)) {
            $post_data['meta_input'][$urlField] = $img_url;
          }
        }
      }
    }
  }
  static protected function do_mapping_taxonomy($data, $option, &$post_data = array()) {
    $meta_mapping = get_option($option, array());
    foreach($meta_mapping as $tax => $def) {
      if(empty($def[Main::KEY_META_KEY]) && !is_int($tax)) {
        $def[Main::KEY_TAXONOMY_NAME] = $tax;
      }
      if(empty($def[Main::KEY_META_KEY]) && empty($def[Main::KEY_TAXONOMY_NAME]) ) {
        continue;
      }
      if(!empty($def[Main::KEY_META_KEY]) && empty($def[Main::KEY_CREATE_TAXONOMY])) {
        continue;
      }
      if(!isset($def[Main::KEY_META_KEY]) && empty($def[Main::KEY_ENABLED])) {
        continue;//these are the taxonomies that are not enabled
      }
      $values = self::has_some_value($data, $def, true, true, true);
      if(!empty($values)) {
        if(isset($def[Main::KEY_META_KEY])) {
          $a = ((array)reset($values));
          $post_data['meta_input'][$def[Main::KEY_META_KEY]] = $a[0];
        } else {
          $post_data['tax_input'][$def[Main::KEY_TAXONOMY_NAME]] = self::flatten_array($values);
        }
      } else if(isset($def[Main::KEY_META_KEY])) {
        $post_data['meta_input'][$def[Main::KEY_META_KEY]] = '';
      }
    }
  }
  //Attachment handing functions
  static protected function image_from_url($url) {
    $existing_image = self::existing_image_by_url($url);
    if(!empty($existing_image)) {
      return $existing_image;
    }
    //We have not seen this image yet, download
    $basename = preg_replace('/[^a-zA-Z0-9_\.]+/', '-', sanitize_file_name(urldecode(basename($url))));
    $upl = wp_upload_dir(gmdate('Y/m'));
    $dn = $upl['path'].'/'.$basename;
    $ext = array_shift(array_reverse(explode('.', $basename)));
    while(file_exists($dn)) {
      $dn = tempnam($upl['path'].'/', str_replace('.'.$ext, '', $basename)).'.'.$ext;
    }
    self::download_from_url($url, $dn);

    if(!file_exists($dn)) {
      return array(null, null);
    }
    return self::create_attachment($dn, $url, $upl['url']);
  }
  static protected function existing_image_by_url($url) {
    $attachments = get_posts(array('post_type' => 'attachment', 'meta_key' => Main::META_ATTACHMENT_DOWNLOADED_FROM, 'meta_value' => $url));
    if(!empty($attachments)) {
      return array($attachments[0]->ID, wp_get_attachment_url($attachments[0]->ID));
    }

    global $wpdb;
    $attachment = array();
    if(strlen($url) < 15) {
      $attachment = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE guid LIKE '%%%s';", $url ));
    }
    if(!empty($attachment)) {
      return array($attachment[0]['id'], wp_get_attachment_url($attachment[0]['id']));
    }
    return false;
  }
  static protected function create_attachment($filename, $url, $url_prefix) {
    $basename = sanitize_file_name(basename($filename));
    $type = wp_check_filetype($filename);
    if(empty($type['type']) && function_exists('exif_imagetype')) {
      $img_type = exif_imagetype($filename);
      switch($img_type) {
        case IMAGETYPE_JPEG: $type['type'] = 'image/jpeg'; break;
        case IMAGETYPE_PNG: $type['type'] = 'image/png'; break;
        case IMAGETYPE_GIF: $type['type'] = 'image/gif'; break;
      }
    } else if (function_exists('getimagesize')) {
      $type['type'] = getimagesize($filename)['mime'];
    }
    if(empty($type['type'])) {
      return array(null, null);
    }
    $aid = wp_insert_attachment(array('post_title' => $basename, 'post_content' => '', 'post_status' => 'publish', 'meta_input' => array(Main::META_ATTACHMENT_DOWNLOADED_FROM => $url), 'post_mime_type' => $type['type']), $filename);
    if(is_wp_error($aid)) {
      return array(null, null);
    }
    wp_update_attachment_metadata( $aid,  wp_generate_attachment_metadata($aid, $filename) );
    return array($aid, $url_prefix.$basename);
  }
  static protected function download_from_url($url, $store_to) {
    if(function_exists('download_url')) {
      $fn = download_url($url);
      if(is_wp_error($fn)) {
        throw new \WpErrorException("File ".$url." failed to download", 0, null, $fn);
        return false;
      } else if (!file_exists($fn)) {
        throw new \Exception("File ".$url." failed to download, missing file");
        return false;
      }
      if(rename($fn, $store_to)) {
        return true;
      }
      throw new \Exception("File ".$url." failed to rename to uploads directory");
      return false;
    } else {
      $cnt = file_get_contents($url);
      if(empty($cnt)) {
        throw new \Exception("File ".$url." failed to download, #2");
        return false;
      }
      file_put_contents($store_to, $cnt);
    }
    return true;
  }

  //Output functions
  static protected function result_error($errors, $function, $filter_data = array(), $code = 400) {
    return apply_filters(Main::ACTION_PREFIX.'_'.$function.'_error', array(
      'status' => 'error',
      'code' => $code,
      'errors' => $errors,
    ), $filter_data);
  }
  static protected function result_success($data, $function, $filter_data = array(), $code = 200) {
    return apply_filters(Main::ACTION_PREFIX.'_'.$function.'_success', ((array)$data) + array_filter(array(
      'status' => 'success',
      'code' => $code,
      'warnings' => self::$warnings,
    )), $filter_data);
  }
  //Retrieve value from input
  static protected function has_some_value($data, $def, $check_validity = false, $add_defaults = false, $add_parents = false, &$has_some_value = false) {
    $out = array();
    foreach((array)$def[Main::KEY_FIELDS] as $field) {
      if(empty($field)) continue;
      if(isset($data[$field])) {
        if(!$has_some_value) {
          $has_some_value = (count(array_filter((array)$data[$field])) > 0);
        }
        if($check_validity) {
          $is_valid = self::only_valid_items($data[$field], $def);
          if(!empty($is_valid)) {
            $out[] = $is_valid;
          }
        } else {
          $out[] = $data[$field];
        }
      }
    }
    if($add_defaults && empty($out) && !empty($def[Main::KEY_DEFAULT_ENABLED]) && !empty($def[Main::KEY_DEFAULT_VALUE])) {
      $out[] = $def[Main::KEY_DEFAULT_VALUE]; // Should this be done only on inserts, how?!
    }
    if($check_validity && $add_parents && !empty($out)) {
      foreach($out as &$arr) {
        if(is_array($arr)) {
          foreach($arr as &$itm) {
            self::add_parents($itm, $arr, $def);
          }
        }
      }
    }
    return empty($out) ? false : $out;
  }
  static protected function add_parents($currentItem, &$arr, $def) {
    $tax = isset($def[Main::KEY_META_KEY]) ? $def[Main::KEY_META_KEY] : $def[Main::KEY_TAXONOMY_NAME];
    if(!empty($def[Main::KEY_META_KEY])) {
      $tax = self::meta_taxonomy_name($def);
    }
    $search_by = self::search_by($currentItem, $def);
    $t = get_term_by($search_by, $currentItem, $tax);
    if($t->parent) {
      $arr[] = $t->parent;
      self::add_parents($t->parent, $arr, $def);
    }
  }
  static protected function only_valid_items($values, $def) {
    $out = array();
    $t = null;
    $prev = null;
    foreach((array)$values as $val) {
      if(!empty($def[Main::KEY_SEPARATOR_ENABLE]) && !empty($def[Main::KEY_SEPARATOR])) {
        $items = array_filter(array_map('trim', explode($def[Main::KEY_SEPARATOR], $val)));
      } else {
        $items = (array)$val;
      }
      $parent = $t;
      if(!empty($prev) && count($prev) > 1) {
        self::$warnings[] = implode(', ', (array)$items).' cannot be preceded by multiple values: '.implode(', ', $prev);
      }
      foreach($items as $_val) {
        //We may have multiple items in one level, maybe?
        foreach((array)$_val as $subval) {
          $t = self::is_valid_term($subval, $def, $parent);
        if(!empty($t) && !is_wp_error($t)) {
          $out[] = isset($def[Main::KEY_META_KEY]) ? $t->slug : $t->term_id;
        }
      }
    }
      $prev = $items;
    }
    return $out;
  }
  static protected function has_valid_value($data, $def, $disallow_multiple = false, &$has_value = false) {
    return !empty(self::has_some_value($data, $def, true, false, false, $has_value));
  }
  static protected function is_valid_value($value, $def) {
    if(is_array($value)) {
      foreach($value as $item) {
        $o = self::is_valid_term($item, $def);
        if(!empty($o) && !is_wp_error($o)) {
          return $o;
        }
      }
      return false;
    }
    return self::is_valid_term($value, $def);
  }
  static protected function search_by($nm, $def) {
    $search_by = is_numeric($nm) && empty($def[Main::KEY_TAXONOMY_SEARCH_NAME]) ? 'term_id' : 'name';
    if(empty($def[Main::KEY_ENABLE_CREATE_TERM]) && empty($def[Main::KEY_TAXONOMY_SEARCH_NAME])) {
      if($search_by === 'name') {
        //custom metas are searched by slug
        $search_by = 'slug';
      }
    }
    return $search_by;
  }
  static protected function is_valid_term($nm, $def, $parent = null) {
    $tax = isset($def[Main::KEY_META_KEY]) ? $def[Main::KEY_META_KEY] : $def[Main::KEY_TAXONOMY_NAME];
    if(!empty($def[Main::KEY_META_KEY])) {
      $tax = self::meta_taxonomy_name($def);
    }
    $search_by = self::search_by($nm, $def);
    if(!empty($parent)) {
      $terms = get_terms(array('taxonomy' => $tax, $search_by => $nm, 'parent' => $parent->term_id, 'hide_empty' => false));
      if(!empty($terms) && !is_wp_error($terms)) {
        $t = $terms[0];
      }
    } else {
      $t = get_term_by($search_by, $nm, $tax);
    }
    if(!self::$is_checking_mapping && empty($t) && !empty($def[Main::KEY_ENABLE_CREATE_TERM]) && $search_by === 'name') {
      //create the term
      $inserted = wp_insert_term($nm, $tax, array('parent' => $parent->term_id));
      if(is_wp_error($inserted)) {
        self::$warnings[] = 'Cannot create term '.$nm.' in '.$tax.' ('.implode(', ', $inserted->get_error_messages()).')';
        return null;
      } else if(!empty($inserted['term_id'])) {
        $t = get_term_by('term_id', $inserted['term_id'], $tax);
      }
    }
    return $t;
  }
  //Input checking functions
  static protected function check_login($data) {
    if(isset($data[Main::INPUT_LOGIN_USERNAME]) && isset($data[Main::INPUT_LOGIN_PASSWORD])) {
      //Supress login cookies
      add_filter('send_auth_cookies', 'com\nestdesign\WpJobManagerApi2\ProcessRequest::disable_login_cookie');
      $out = wp_signon(array('user_login' => $data[Main::INPUT_LOGIN_USERNAME], 'user_password' => $data[Main::INPUT_LOGIN_PASSWORD], 'rememberme' => 0), false);
      remove_filter('send_auth_cookies', 'com\nestdesign\WpJobManagerApi2\ProcessRequest::disable_login_cookie');
      if(is_wp_error($out)) {
        return $out;
      }
      wp_set_current_user($out->ID);
      return $out;
    }
    return false;
  }
  static protected function check_required_fields_meta_mappings($data, &$errors = array(), $option = Main::OPTION_META_MAPPING) {
    $meta_mapping = array_filter(get_option($option, array()), function ($def) {
      return !empty($def[Main::KEY_VALID_REQUIRED]);
    });
    if(!empty($meta_mapping)) {
      foreach($meta_mapping as $def) {
        if(empty($def[Main::KEY_CREATE_TAXONOMY]) && !self::has_some_value($data, $def)) {
          $errors[] = array('required' => array_filter($def[Main::KEY_FIELDS]));
        }
      }
    }
  }
  static protected function check_required_fields_items_from_taxonomy($data, &$errors = array(), $option = Main::OPTION_META_CUSTOM) {
    $meta_custom = array_filter(get_option($option, array()), function ($def) use ($data) {
      return (isset($def[Main::KEY_META_KEY]) || !empty($def[Main::KEY_ENABLED])) && (!empty($def[Main::KEY_VALID_REQUIRED]) || self::has_some_value($data, $def));
    });
    if(!empty($meta_custom)) {
      foreach($meta_custom as $name => $def) {
        if(!is_numeric($name) && empty($def[Main::KEY_TAXONOMY_NAME])) {
          $def[Main::KEY_TAXONOMY_NAME] = $name;
        }
        $has_value = false;
        if(!empty($def[Main::KEY_VALID_REQUIRED])) {
          if(empty($def[Main::KEY_ENABLE_CREATE_TERM]) && !empty($def[Main::KEY_TAXONOMY_NAME])) {
            if(!self::has_valid_value($data, $def, true, $has_value)) {
              $errors[] = array($has_value ? 'invalid' : 'required' => array_filter((array)$def[Main::KEY_FIELDS]));
            }
          } else {
            $values = self::has_some_value($data, $def);
            if(!$values) {
              $errors[] = array('required' => array_filter((array)$def[Main::KEY_FIELDS]));
            }
          }
        }
      }
    }
  }
  static protected function has_all_required_fields($data) {
    $errors = array();
    self::$is_checking_mapping = true;
    self::check_required_fields_meta_mappings($data, $errors, Main::OPTION_META_MAPPING);
    self::check_required_fields_meta_mappings($data, $errors, Main::OPTION_META_CUSTOM);
    self::check_required_fields_meta_mappings($data, $errors, Main::OPTION_FILES);
    self::check_required_fields_items_from_taxonomy($data, $errors, Main::OPTION_META_CUSTOM);
    self::check_required_fields_items_from_taxonomy($data, $errors, Main::OPTION_EXISTING_TAXONOMIES);
    self::check_required_fields_items_from_taxonomy($data, $errors, Main::OPTION_CREATE_TAXONOMY);
    self::$is_checking_mapping = false;
    return $errors;
  }
  // some item reduce functions
  static protected function list_show_term($term, $show_id = true) {
    if($show_id) {
      return array(
        'name' => $term->name,
        'term_id' => $term->term_id,
        'parent_term_id' => $term->parent,
      );
    }
    return array(
      'name' => $term->name,
      'slug' => $term->slug,
    );
  }
  static protected function report_show_item($post) {
    static $cachedOptions = array();
    $data = array(
      'job_title' => $post->post_title,
      'url' => get_permalink($post),
      'local_id' => $post->ID,
      'remote_id' => get_post_meta($post->ID, Main::META_REMOTE_ID, true),
    );
    foreach(array(Main::OPTION_META_MAPPING, Main::OPTION_META_CUSTOM) as $metaKey) {
      $cachedOptions[$metaKey] = $opts = !empty($cachedOptions[$metaKey]) ? $cachedOptions[$metaKey] : get_option($metaKey);
      if(!empty($opts)) {
        foreach($opts as $item) {
          if(!empty($item[Main::KEY_ENABLE_REPORTS])) {
            $fields = $item[Main::KEY_FIELDS];
            $data[reset($fields)] = get_post_meta($post->ID, $item[Main::KEY_META_KEY], true);
          }
        }
      }
    }
    foreach(array(Main::OPTION_EXISTING_TAXONOMIES, Main::OPTION_CREATE_TAXONOMY) as $taxKey) {
      $cachedOptions[$taxKey] = $opts = !empty($cachedOptions[$taxKey]) ? $cachedOptions[$taxKey] : get_option($taxKey);
      if(!empty($opts)) {
        foreach($opts as $key => $item) {
          if(!empty($item[Main::KEY_ENABLE_REPORTS])) {
            $fields = $item[Main::KEY_FIELDS];
            $termKey = $item[Main::KEY_ENABLE_REPORTS];
            $tax = is_int($key) ? $item[Main::KEY_TAXONOMY_NAME] : $key;
            $data[reset($fields)] = array_map(function ($term) use ($termKey) { return $term->{$termKey}; }, get_the_terms($post->ID, $tax));
          }
        }
      }
    }
    return $data;
  }
  //Misc functions
  static public function disable_login_cookie() {
    return false;
  }
  static protected function search_post_arguments($user, $remote_id = null, $data = array()) {
    $args = array(
      'post_type' => Main::currentPostType(),
      'author' => apply_filters(Main::ACTION_PREFIX.'_author_id', $user->ID, $data),
      'suppress_filters' => 1,
      'order' => 'ASC',//The first one is hopefully the original
      'posts_per_page' => -1,
    );
    if(!empty($remote_id)) {
      $args['meta_key'] = Main::META_REMOTE_ID;
      $args['meta_value'] = $remote_id;
    }
    return $args;
  }
  static protected function find_job_by_remoteid($remote_id, $user, $data) {
    if(empty($remote_id)) {
      return false;
    }
    //let's try to find the post
    $query_params = self::search_post_arguments($user, $remote_id, $data);
    $allow_option_user_use_email = get_option(Main::OPTION_USER_USE_EMAIL, false);
    if(!empty($allow_option_user_use_email)) {
      //As we can assign the job to any user based on the email, we search through the whole DB
      unset($query_params['author']);
    }
    $the_query = new \WP_Query( $query_params );
    if ($the_query->have_posts()) {
      $the_query->the_post();
      return apply_filters(Main::ACTION_PREFIX.'_'.__FUNCTION__.'_found_existing_post', get_the_ID(), $remote_id, $data, $post_data, $the_query);
    }
    return false;
  }
  static protected function flatten_array($values, &$out = array()) {
    foreach($values as $itm) {
      if(is_array($itm)) {
        self::flatten_array($itm, $out);
      } else if(!empty($itm)) {
        $out[] = $itm;
      }
    }
    return $out;
  }
  static public function endpoints() {
    $methods = array();
    foreach(get_class_methods(__CLASS__) as $fn_name) {
      if(substr($fn_name, 0, 4) === 'API_') {
        $methods[substr($fn_name, 4)] = $fn_name;
      }
    }
    return $methods;
  }
  static protected function get_post_status($id) {
    return get_post_status($id);
  }
  static protected function wp_insert_post($post_data) {
    return wp_insert_post($post_data);
  }
  static protected function wp_delete_post($id, $force) {
    return wp_delete_post($id, $force);
  }
}
ProcessRequest::init();
