<?php
namespace com\nestdesign\WpJobManagerApi2;
class Settings {
  use MetaTaxonomyTrait;
  use CreateTermTrait;
  static public $main_post_meta = array('post_title', 'post_content', 'excerpt', 'post_status');
  static public function init() {
    self::registerWPJBActions();
    self::registerWPJBFieldActions();
    self::registerWPJBAjax();
    self::addAdminJavascript();
    $map = get_option(Main::OPTION_META_MAPPING);
    if(!empty($map)) {
      foreach($map as $item) {
        if($item[Main::KEY_META_KEY] === 'excerpt') {
          add_post_type_support('job_listing', 'excerpt');
        }
      }
    }
    if(Main::isSimpleJobBoard()) {
      self::addSimpleJobBoardMapping();
    }
    if(Main::isNooJob()) {
      self::addNooJobMapping();
    }
  }
  static protected function addNooJobMapping() {
    static $jb_settings = array();
    self::loadSettings($jb_settings);
    add_filter('noo_job_settings_tabs_array', function ($tabs) {
      $tabs['wpjb_api'] = __('API2', 'wp-job-manager-api2');
      return $tabs;
    }, 99 );
    foreach($jb_settings as $key=>$cfg) {
      foreach($cfg[1] as $item) {
        register_setting( $key, $item['name'], array('default' => $cfg['std']));
      }
    }
    add_filter('whitelist_options', function ($opts) use ($jb_settings) {
      foreach($jb_settings as $key => $cfg) {
        $opts[$key] = array_column($cfg[1], 'name');
      }
      return $opts;
    });
    add_action('noo_job_setting_wpjb_api', function () use ($jb_settings) {
      foreach($jb_settings as $key=>$cfg) {
        settings_fields($key);
        echo '<div id="settings-'.$key.'" class="sjb-admin-settings wpjb-settings-wrapper"><form action="options.php" method="post" id="'.$key.'">';
        echo settings_fields($key);
        echo self::renderSettings($cfg[1], $key);
        echo '</div>';
      }
    });
  }
  static protected function addSimpleJobBoardMapping() {
    static $jb_settings = array();
    self::loadSettings($jb_settings);
    //We need to set the remote id as required aditional field
    foreach($jb_settings as $key=>$cfg) {
      foreach($cfg[1] as $item) {
        register_setting( $key, $item['name'], array('default' => $cfg['std']));
      }
    }
    add_filter('whitelist_options', function ($opts) use ($jb_settings) {
      foreach($jb_settings as $key => $cfg) {
        $opts[$key] = array_column($cfg[1], 'name');
      }
      return $opts;
    });
    add_filter('sjb_settings_tab_menus', function ($tabs) use ($jb_settings) {
      foreach($jb_settings as $key=>$cfg) {
        $tabs[$key] = $cfg[0];
      }
      return $tabs;
    }, 150);
    add_action('sjb_settings_tab_section', function () use ($jb_settings) {
      foreach($jb_settings as $key=>$cfg) {
        echo '<div id="settings-'.$key.'" class="sjb-admin-settings wpjb-settings-wrapper" style="display: none;"><form action="options.php" method="post" id="'.$key.'">';
        echo settings_fields($key);
        echo self::renderSettings($cfg[1], $key);
        echo '<input type="submit" name="upload_file_form_submit" id="upload-file-form-submit" class="button button-primary" value="'.esc_html__('Save Changes', 'simple-job-board').'" />';
        echo '</div>';
      }
    }, 10);
//     add_action('sjb_save_setting_sections', function () use ($jb_settings) {
//       self::loadSettings($jb_settings);
//       foreach($jb_settings as $key => $cfg) {
//         //$val = filter_input( INPUT_POST, $key, FILTER_DEFAULT);
//         //set_option($key, $val);
//       }
//       if(!empty($_POST)) {
//         die('adsfgh');
//       }
//     });
  }
  static protected function renderSettings($config, $key) {
    ob_start();
    $fields = array_column(array_map(function ($cfg) {
      if(strpos($cfg['type'], 'wp_job_manager_api2_') === 0) {
        $cfg['action'] = $cfg['type'];
        $cfg['type'] = 'action';
        $cfg['action_prefix'] = 'wp_job_manager_admin_field_';
      }
      return $cfg;
    }, $config), null, 'name');
    foreach($fields as $key => $cfg) {
      $opt = get_option($cfg['name']);
      if(!empty($opt)) {
        $data[$key] = $opt;
      } else if(!isset($data[$key]) && isset($cfg['std']))
        $data[$key] = $cfg['std'];
    }
    self::child_display_fields($fields, $data);
    return ob_get_clean();
  }
  static protected function loadSettings(&$jb_settings = array()) {
    if(empty($jb_settings)) {
      $jb_settings = apply_filters('job_manager_settings', $jb_settings);
    }
  }
  static protected function registerWPJBActions() {
    add_filter('job_manager_settings', function ($settings) {
      $settings['wp_job_manager_api2'] = array(
        __( 'API2', 'wp-job-manager-api2' ),
        array(
          array(
            'name'  => Main::OPTION_EXISTING_TAXONOMIES,
            'label' => __( 'Existing Taxonomies', 'wp-job-manager-api2' ),
            'std'   => array(
              'job_listing_category' => array('enabled' => 1, Main::KEY_ENABLE_CREATE_TERM => 1, Main::KEY_ENABLE_CREATE_TERM => 1, 'valid_required' => 1, Main::KEY_FIELDS => 'category_name'),
              'job_listing_region' => array('enabled' => 1, Main::KEY_ENABLE_CREATE_TERM => 1, Main::KEY_ENABLE_CREATE_TERM => 1, 'valid_required' => 1, Main::KEY_FIELDS => 'location_name'),
              'job_listing_type' => array('enabled' => 1, Main::KEY_ENABLE_CREATE_TERM => 1, Main::KEY_ENABLE_CREATE_TERM => 0, 'valid_required' => 1, Main::KEY_FIELDS => 'job_type'),
            ),
            'type'  => 'wp_job_manager_api2_existing_taxonomies',
            'desc' => __('Default matching is to slug, Taxonomies with Enable create term enabled will be matched to name. These can have multiple term-paths, when there are multiple input key fields defined or when IDs are supplied.', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_CREATE_TAXONOMY,
            'label' => __( 'Create Taxonomies', 'wp-job-manager-api2' ),
            'std'   => array(),
            'type'  => 'wp_job_manager_api2_create_taxonomies',
            'desc' => __('Default matching is to slug, Taxonomies with Enable create term enabled will be matched to name. These can have multiple term-paths, when there are multiple input key fields defined or when IDs are supplied.', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_META_MAPPING,
            'label' => __( 'Metadata Mapping', 'wp-job-manager-api2' ),
            'std'   => array(
              array(Main::KEY_META_KEY => 'post_title', Main::KEY_FIELDS => array('title'), Main::KEY_VALID_REQUIRED => '1'),
              array(Main::KEY_META_KEY => 'post_content', Main::KEY_FIELDS => array('desc_long'), Main::KEY_VALID_REQUIRED => '1'),
              array(Main::KEY_META_KEY => 'excerpt', Main::KEY_FIELDS => array('desc_short')),
              array(Main::KEY_META_KEY => '_contact_name', Main::KEY_FIELDS => array('contact_name')),
              array(Main::KEY_META_KEY => Main::META_APPLICATION, Main::KEY_FIELDS => array('contact_email')),
              array(Main::KEY_META_KEY => '_remote_id', Main::KEY_FIELDS => array('job_id'), Main::KEY_VALID_REQUIRED => '1'),
              array(Main::KEY_META_KEY => '_ref_key', Main::KEY_FIELDS => array('ref')),
              array(Main::KEY_META_KEY => '_job_location', Main::KEY_FIELDS => array('location_text')),
              array(Main::KEY_META_KEY => '_featured', Main::KEY_FIELDS => array('featured')),
              array(Main::KEY_META_KEY => '_salary_min', Main::KEY_FIELDS => array('salary')),
              array(Main::KEY_META_KEY => '_salary_max', Main::KEY_FIELDS => array('salary_max')),
              array(Main::KEY_META_KEY => '_salary_negotiable', Main::KEY_FIELDS => array('salary_negotiable')),
              array(Main::KEY_META_KEY => '_company_video', Main::KEY_FIELDS => array('video_url')),
            ),
            'type'  => 'wp_job_manager_api2_meta_mapping',
            'desc' => __('Maps input keys to existing meta fields created by other plugins, only first received field value would be used', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_META_CUSTOM,
            'label' => __( 'Custom Metadata', 'wp-job-manager-api2' ),
            'std'   => array(
              array(Main::KEY_META_KEY => '_salary_currency', Main::KEY_CREATE_TAXONOMY => 1, 'singular' => 'Currency', 'plural' => 'Currencies', Main::KEY_FIELDS => array('salary_currency')),
              array(Main::KEY_META_KEY => '_salary_frequency', Main::KEY_CREATE_TAXONOMY => 1, 'singular' => 'Salary Frequency', 'plural' => 'Salary Frequencies', Main::KEY_FIELDS => array('salary_frequency')),
            ),
            'type'  => 'wp_job_manager_api2_meta_custom',
            'desc' => __('Allows you to create custom meta fields, including the ability to require one of set values based on custom taxonomy', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_FILES,
            'label' => __( 'Files', 'wp-job-manager-api2' ),
            'type'  => 'wp_job_manager_api2_files',
            'std' => array(
              array(Main::KEY_ATTACHMENT_ID_FIELDS => array(Main::META_ATTACHMENT_IMPORT), Main::KEY_FIELDS => array('image')),
            ),
            'desc' => __('Files currently supports URL as input value. Either URL for existing media file, or the URL would be downloaded for later usage.', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_STATUS_ON_INSERT,
            'label' => __( 'Post status on insert', 'wp-job-manager-api2' ),
            'std'      => 'publish',
            'type'     => 'select',
            'options' => get_post_stati(),
            'desc' => __('If you do not want to make jobs public immediately, choose diferrent staus than publish. This settings would be ignored if you define meta mapping for Post status and there is valid value in the input', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_USER_USE_EMAIL,
            'label' => __( 'Attach the job to user based on the application email', 'wp-job-manager-api2' ),
            'std'      => '0',
            'type'     => 'checkbox',
            'cb_label' => 'Enable',
            'desc' => __('Only enable when you trust the senders! When enabled, the job will be attached to a user which matches the _application meta key, otherwise it would be attached to the user which was supplied in the login fields.', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_NEW_DATE_ON_UPDATE,
            'label' => __( 'Update item post date on update', 'wp-job-manager-api2' ),
            'std'      => '1',
            'type'     => 'checkbox',
            'cb_label' => 'Enable',
            'desc' => __('When enabled each uploadJob request would change the post date', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_ENABLE_APPLICATION_NOTIFICATION,
            'label' => __( 'Enable Application Notification', 'wp-job-manager-api2' ),
            'std'      => '1',
            'type'     => 'checkbox',
            'cb_label' => 'Enable',
            'desc' => __('The POSTed data depends on the apply module used and the fields available in the apply form', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_ENABLE_APPLICATION_SOURCE_TRACK,
            'label' => __( 'Enable Application Source Tracking', 'wp-job-manager-api2' ),
            'std'      => '1',
            'type'     => 'checkbox',
            'cb_label' => 'Enable',
            'desc' => __('The POSTed data will include the source URL from where the user entered the website (if available). This may be stored as cookie variable \''.Main::KEY_APPLICATION_SOURCE_TRACK.'\', if session is not available at the time.', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_ENABLE_APPLICATION_IN_JSON,
            'label' => __( 'Application Tracking in JSON', 'wp-job-manager-api2' ),
            'std'      => '1',
            'type'     => 'checkbox',
            'cb_label' => 'Enable',
            'desc' => __('The data will be POSTed in a JSON format', 'wp-job-manager-api2'),
          ),
          array(
            'name' => Main::OPTION_IP_FILTER,
            'label' => __( 'IP Filter', 'wp-job-manager-api2' ),
            'std' => array(),
            'type' => 'wp_job_manager_api2_ip_filter',
            'desc' => __('When enabled only specified ips/ranges would be allowed to access the API', 'wp-job-manager-api2'),
          ),
          array(
            'name' => Main::OPTION_JOB_URL,
            'label' => __( 'Job URL', 'wp-job-manager-api2' ),
            'std' => array(),
            'type' => 'wp_job_manager_api2_url_format',
            'desc' => __('Specify the URL format', 'wp-job-manager-api2'),
          ),
          array(
            'name' => Main::OPTION_LOG_REQUESTS,
            'label' => __( 'Log Requests', 'wp-job-manager-api2' ),
            'std' => Main::VALUE_LOG_REQUESTS_NONE,
            'type' => 'select',
            'options' => array(
              Main::VALUE_LOG_REQUESTS_NONE => __('No requests', 'wp-job-manager-api2'),
              Main::VALUE_LOG_REQUESTS_FAILED => __('Failed requests', 'wp-job-manager-api2'),
              Main::VALUE_LOG_REQUESTS_ALL => __('All requests', 'wp-job-manager-api2')
            ),
            'desc' => __('When set to something else than none, this will output some debuging data from received API requests to the wp-content/uploads directory.', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => Main::OPTION_PLUGIN_CLEANUP,
            'label' => __( 'Delete config on plugin uninstall', 'wp-job-manager-api2' ),
            'std'      => '1',
            'type'     => 'checkbox',
            'cb_label' => 'Enable',
            'desc' => __('When enbled, the changes in the default settings cannot be recovered after plugin deactivation, unless you have the export file', 'wp-job-manager-api2'),
          ),
          array(
            'name'  => 'wp_job_manager_api2_restore',
            'label' => __( 'Restore settings', 'wp-job-manager-api2' ),
            'type' => 'wp_job_manager_api2_restore',
            'desc' => __('This allows you to grab the settings and use it on another site right away.', 'wp-job-manager-api2'),
          )
        )
      );
      return $settings;
    }, 10, 1);
  }
  static protected function registerWPJBFieldActions() {
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_existing_taxonomies', function ($option, $attributes, $value, $placeholder = null) {
      global $wp_taxonomies;
      $taxes = array();
      foreach($wp_taxonomies as $key=>$tax) {
        if(in_array(Main::currentPostType(), $tax->object_type))
          $taxes[] = $key;
      }
      $mytaxes = array_filter(array_map('self::meta_taxonomy_name', get_option(Main::OPTION_META_CUSTOM, array())));
      $customtaxes = array_column(get_option(Main::OPTION_CREATE_TAXONOMY, array()), Main::KEY_TAXONOMY_NAME);
      echo '<ul class="'.Main::ACTION_PREFIX.'-container">';
      $data = empty($value) ? array() : self::array_filter_deep($value);
      global $wp_taxonomies;
      foreach($taxes as $tax) {
        if(in_array($tax, $mytaxes, true)) continue;
        if(in_array($tax, $customtaxes, true)) continue;
        $val = $data[$tax];
        $name_prefix = esc_attr($option['name']).'['.esc_attr($tax).']';
        echo '<li class="taxonomy-item"><div class="wpjbapi2-container"><label title="'.esc_html__('Enable / Disable the usage of the taxonomy in the API', 'wp-job-manager-api2').'"><input type="checkbox" value="1" name="'.$name_prefix.'[enabled]" class="field-enabled" '.(!empty($val['enabled']) ? ' checked="checked"' : '').' />'.$tax.'</label></div>';
        echo self::taxonomy_default_fields($val, $name_prefix);
        echo '</li>';
      }
      echo '</ul>';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_create_taxonomies', function ($option, $attributes, $value, $placeholder = null) {
      $data = empty($value) ? array() : self::array_filter_deep($value);
      $data[] = array();
      $i = 0;
      echo '<ul class="'.Main::ACTION_PREFIX.'-container">';
      foreach($data as $tax => $def) {
        $name_prefix = esc_attr($option['name']).'['.$i.']';
        echo '<li class="taxonomy-item">';
        $field_name = 'enabled';
        echo '<label class="lbl-'.$field_name.' type-checkbox"><div class="wpjbapi2-container"><input type="checkbox" class="field-'.$field_name.'" name="'.$name_prefix.'['.$field_name.']" value="1" '.(is_int($tax) && !empty($def[$field_name]) ? ' checked="checked"' : '').'/>'.esc_html__('Enable', 'wp-job_manager-api2').'</label></div>';
        echo self::create_taxonomy_fields($def, $name_prefix);
        echo self::taxonomy_default_fields($def, $name_prefix);
        echo '</li>';
        $i ++;
      }
      echo '</ul>';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_meta_mapping', function ($option, $attributes, $value, $placeholder = null) {
      $defaultReportsEnable = array('post_title', Main::META_REMOTE_ID);
      list($metas, $fields_wpjm) = self::available_meta_fields(true);
      //add some basic fields (post content, excerpt, title)
      foreach(array_reverse(self::$main_post_meta) as $key) {
        array_unshift($metas, $key);
      }
      $data = !empty($value) ? self::array_filter_deep($value) : array();
      $data[] = array();
      $i = 0;
      //we should make sure we have the _remote_id always in the settings
      echo '<ul class="'.Main::ACTION_PREFIX.'-container">';
      $used_options = array();
      $has_remote_id = false;
      foreach($data as $item) {
        $name_prefix = esc_attr($option['name']).'['.$i.']';
        $val = isset($item[Main::KEY_META_KEY]) ? $item[Main::KEY_META_KEY] : null;
        if(!empty($val) && in_array($val, $used_options)) {
          continue;
        }
        $used_options[] = $val;
        echo '<li class="itm-mapping"><select name="'.$name_prefix.'[meta_key]" class="field-meta_key"><option value="">'.esc_html__('-- select --', 'wp-job-manager-api2').'</option>';
        echo self::meta_options($val, $metas, $fields_wpjm);
        echo '</select>';
        echo '<span class="meta-details">';
        echo self::checkbox_valid_required($item, $name_prefix, !in_array($item[Main::KEY_META_KEY], self::$main_post_meta, true) && $item[Main::KEY_META_KEY] !== Main::META_REMOTE_ID);
        echo '<div class="itm-'.Main::KEY_ENABLE_REPORTS.' type-checkbox wpjbapi2-container"><label><input type="checkbox"'.(in_array($val, $defaultReportsEnable) ? ' disabled="disabled"' : '').' name="'.$name_prefix.'['.Main::KEY_ENABLE_REPORTS.']" value="1"'.(in_array($val, $defaultReportsEnable) || !empty($item[Main::KEY_ENABLE_REPORTS]) ? ' checked="checked"' : '').' />'.esc_html__('Show in Active Reports', 'wp-job-manager-api2').'</label></div>';
        if($item[Main::KEY_META_KEY] === Main::META_REMOTE_ID && count(array_filter($fields)) > 0 && !empty($item[Main::KEY_VALID_REQUIRED])) {
          $has_remote_id = true;
        }
        $fields = !empty($item[Main::KEY_FIELDS]) ? array_filter($item[Main::KEY_FIELDS]) : array();
        $fields[] = '';
        echo self::get_input_fields($name_prefix, $fields);
        echo '</span></li>';
        $i ++;
      }
      echo '</ul>';
      if(!$has_remote_id) {
        echo '<p class="error">'.sprintf(esc_html__('You need to setup the field %s and it must be set as required', 'wp-job-manager-api2'), (isset($fields_wpjm[Main::META_REMOTE_ID]['label']) ? $fields_wpjm[Main::META_REMOTE_ID]['label'] : ucfirst(trim(str_replace('_', ' ', Main::META_REMOTE_ID))))).'</p>';
      }
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_meta_custom', function ($option, $attributes, $value, $placeholder = null) {
      $data = !empty($value) ? self::array_filter_deep($value) : array();
      $data[] = array();
      $i = 0;
      echo '<ul class="'.Main::ACTION_PREFIX.'-container">';
      foreach($data as $item) {
        $name_prefix = esc_attr($option['name']).'['.$i.']';
        echo '<li class="itm-custmapping">';
        echo '<input class="field-meta_key" type="text" name="'.$name_prefix.'[meta_key]" value="'.(!empty($item[Main::KEY_META_KEY]) ? esc_attr($item[Main::KEY_META_KEY]) : '').'" placeholder="'.esc_html__('Meta key, should start with _', 'wp-job-manager-api2').'" />';
        echo '<span class="meta-details">';
        echo self::checkbox_valid_required($item, $name_prefix, true);
        echo '<span class="wpjbapi2-container container-'.Main::KEY_VALID_REQUIRED.'">';
        echo '<label class="field-'.Main::KEY_CREATE_TAXONOMY.'"><input class="field-'.Main::KEY_CREATE_TAXONOMY.'" type="checkbox" name="'.$name_prefix.'['.Main::KEY_CREATE_TAXONOMY.']" value="1"'.(!empty($item[Main::KEY_CREATE_TAXONOMY]) ? ' checked="checked"' : '').' />'.esc_html__('Create Taxonomy', 'wp-job-manager-api2').'</label>';
        echo '<span class="meta-'.Main::KEY_CREATE_TAXONOMY.'"'.(empty($item[Main::KEY_CREATE_TAXONOMY]) ? ' style="display: none"' : '').'>';
        echo self::create_taxonomy_fields($item, $name_prefix, array(Main::KEY_TAXONOMY_NAME));
        echo '<a href="edit-tags.php?taxonomy='.self::meta_taxonomy_name($item).'&post_type='.Main::currentPostType().'">'.esc_html__('Edit', 'wp-job-manager-api2').'</a>';
        echo '</span>';
        echo '<span class="meta-'.Main::KEY_META_TITLE.'"'.(!empty($item[Main::KEY_CREATE_TAXONOMY]) ? ' style="display: none"' : '').'>';
        echo '<input type="text" name="'.$name_prefix.'['.Main::KEY_META_TITLE.']" value="'.(isset($item[Main::KEY_META_TITLE]) ? htmlspecialchars($item[Main::KEY_META_TITLE], ENT_QUOTES) : '').'" placeholder="'.esc_html__('Title', 'wp-job-manager-api2').'" title="'.esc_html__('Used on the job edit page', 'wp-job-manager-api2').'" />';
        echo '</span>';
        $fields = !empty($item[Main::KEY_FIELDS]) ? $item[Main::KEY_FIELDS] : array('');
        echo self::get_input_fields($name_prefix, $fields, false);
        echo '</span></li>';
        $i++;
      }
      echo '</ul>';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_restore', function ($option, $attributes, $value, $placeholder = null) {
      echo '<a class="'.Main::ACTION_PREFIX.'_button_export" href="#" onclick="window.open(ajaxurl+\'?action='.Main::ACTION_PREFIX.'_export_settings\');return false;">'.esc_html__('Export', 'wp-job-manager-api2').'</a><input type="file" id="setting-'.$option['name'].'" onchange="this.nextSibling.style.display = this.files.length > 0 ? \'initial\' : \'none\';" /><input style="display: none;" type="button" class="'.Main::ACTION_PREFIX.'_import_settings" value="'.esc_html__('Import', 'wp-job-manager-api2').'" data-wpnonce="'.esc_attr(wp_create_nonce( Main::ACTION_PREFIX.'_import_settings')).'" />';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_files', function ($option, $attributes, $value, $placeholder = null) {
      list($metas, $fields_wpjm) = self::available_meta_fields(true);
      //add some basic fields (post content, excerpt, title)
      $data = !empty($value) ? self::array_filter_deep($value) : array();
      $data[] = array();
      $i = 0;
      //we should make sure we have the _remote_id always in the settings
      echo '<ul class="'.Main::ACTION_PREFIX.'-container">';
      $used_options = array();
      $has_remote_id = false;
      foreach($data as $item) {
        $name_prefix = esc_attr($option['name']).'['.$i.']';
        $val = isset($item[Main::KEY_META_KEY]) ? $item[Main::KEY_META_KEY] : null;
        if(!empty($val) && in_array($val, $used_options)) {
          continue;
        }
        $used_options[] = $val;
        echo '<li class="itm-mapping">';
        echo self::checkbox_valid_required($item, $name_prefix, true);
        $fields = !empty($item[Main::KEY_FIELDS]) ? array_filter($item[Main::KEY_FIELDS]) : array();
        $fields[] = '';
        echo self::get_input_fields($name_prefix, $fields);
        $ids = !empty($item[Main::KEY_ATTACHMENT_ID_FIELDS]) ? $item[Main::KEY_ATTACHMENT_ID_FIELDS] : array();
        $ids[] = '';
        echo self::get_multiple_meta_fields($name_prefix.'['.Main::KEY_ATTACHMENT_ID_FIELDS.']', $ids, $metas, $fields_wpjm, __('Put ID to', 'wp-job-manager-api2'));
        echo self::get_file_custom_meta_inputs($name_prefix.'['.Main::KEY_FILE_CUSTOM_ID_FIELD.']', isset($item[Main::KEY_FILE_CUSTOM_ID_FIELD]) ? $item[Main::KEY_FILE_CUSTOM_ID_FIELD] : array(), __('Enable custom ID meta', 'wp-job-manager-api2'));
        $urls = !empty($item[Main::KEY_ATTACHMENT_URL_FIELDS]) ? $item[Main::KEY_ATTACHMENT_URL_FIELDS] : array();
        $urls[] = '';
        echo self::get_multiple_meta_fields($name_prefix.'['.Main::KEY_ATTACHMENT_URL_FIELDS.']', $urls, $metas, $fields_wpjm, __('Put URL to', 'wp-job-manager-api2'));
        echo self::get_file_custom_meta_inputs($name_prefix.'['.Main::KEY_FILE_CUSTOM_URL_FIELD.']', isset($item[Main::KEY_FILE_CUSTOM_URL_FIELD]) ? $item[Main::KEY_FILE_CUSTOM_URL_FIELD] : array(), __('Enable custom URL meta', 'wp-job-manager-api2'));
        echo '</li>';
        $i ++;
      }
      echo '</ul>';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_ip_filter', function ($option, $attributes, $value, $placeholder = null) {
      $field_start = '<input type="text" name="'.$option['name'].'[ips][]" class="copyable" placeholder="1.2.3.4 or 1.2.3.4/32" value="';
      $ips = !empty($value['ips']) ? array_filter(array_map('trim', $value['ips'])) : array();
      $ips[] = '';
      echo '<div class="enabled-parent '.Main::ACTION_PREFIX.'-container"><label><input type="checkbox" class="field-'.Main::KEY_ENABLED.'" id="setting-'.$option['name'].'" name="'.$option['name'].'['.Main::KEY_ENABLED.']"'.(!empty($value[Main::KEY_ENABLED]) ? ' checked="checked"' : '').' value="1" />'.esc_html__('Enable IP filter', 'wp-job-manager-api2').'</label><span class="enabled-content">'.$field_start.implode('" />'.$field_start, array_map('esc_attr', $ips)).'" /></span></div>';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
    add_action('wp_job_manager_admin_field_wp_job_manager_api2_url_format', function ($option, $attributes, $value, $placeholder = null) {
      $selects = array();
      $value['fields'] = !empty($value['fields']) ? array_filter($value['fields']) : array();
      $value['fields'][] = '';
      list($metas, $fields_wpjm) = self::available_meta_fields(true);
      foreach(array_reverse(self::$main_post_meta) as $key) {
        array_unshift($metas, $key);
      }
      foreach($value['fields'] as $val) {
        $field = '<select name="'.$option['name'].'[fields][]" class="copyable"><option value="">'.esc_html__('-- select --', 'wp-job-manager-api2').'</option>';

        //Taxonomies
        $field .= '<optgroup label="Taxonomies">';
        global $wp_taxonomies;
        $taxes = array();
        foreach($wp_taxonomies as $key=>$tax) {
          if(in_array(Main::currentPostType(), $tax->object_type))
            $field .= '<option value="'.$tax->name.'"'.($val === $tax->name ? ' selected="selected"' : '').'>'.$tax->labels->singular_name.'</option>';
        }
        $field .= '</optgroup>';
        //Meta fields
        $field .= '<optgroup label="Meta fields"><option ';
        $field .= implode('</option><option ', array_map(function ($meta) use ($val) {
          return 'value="'.htmlspecialchars($meta, ENT_QUOTES).'"'.($val === $meta ? ' selected="selected"' : '').'>'.htmlspecialchars(ucfirst(trim(str_replace('_', ' ', $meta))));
        }, $metas));
        $field .= '</option></optgroup></select>';
        $selects[] = $field;
      }
      $ips = !empty($value['ips']) ? array_filter(array_map('trim', $value['ips'])) : array();
      $ips[] = '';
      echo '<div class="enabled-parent '.Main::ACTION_PREFIX.'-container"><label><input type="checkbox" class="field-'.Main::KEY_ENABLED.'" id="setting-'.$option['name'].'-'.Main::KEY_ENABLED.'" name="'.$option['name'].'['.Main::KEY_ENABLED.']"'.(!empty($value[Main::KEY_ENABLED]) ? ' checked="checked"' : '').' value="1" />'.esc_html__('Enable Job URL Format', 'wp-job-manager-api2').'</label><div class="enabled-content"><label><input type="checkbox" class="field-only_new" id="setting-'.$option['name'].'-only_new" name="'.$option['name'].'[only_new]"'.(!empty($value['only_new']) || empty($value[Main::KEY_ENABLED]) ? ' checked="checked"' : '').' value="1" />'.esc_html__('Only for new jobs', 'wp-job-manager-api2').'</label><div class="fields">'.implode('', $selects).'</div><div class="separator"><input type="text" name="'.$option['name'].'[separator]" value="'.htmlspecialchars($value['separator'], ENT_QUOTES).'" placeholder="Separator, eg. - _" title="'.esc_html__('Only characters wordpress accepts in the URL, empty means -, invalid characters will be replaced to -.', 'wp-job-manager-api2').'" /></div></div></div>';
      if(!empty($option['desc'])) {
        echo '<p class="description">'.$option['desc'].'</p>';
      }
    }, 10, 4);
  }
  static protected function available_meta_fields($is_files = false){
    $metas = self::get_post_type_meta_keys(Main::currentPostType());
    if($is_files) {
      array_unshift($metas, Main::META_ATTACHMENT_IMPORT);
    }
    array_unshift($metas, Main::META_REMOTE_ID);
    $opt = get_option(Main::OPTION_ENABLE_APPLICATION_NOTIFICATION);
    if($opt === '1') {
      array_push($metas, Main::META_APPLICATION_URL);
    }
    $fields_wpjm = array();
    if(Main::isWPJM()) {
      if(class_exists('\WP_Job_Manager_Writepanels')) {
        $fields_wpjm = \WP_Job_Manager_Writepanels::instance()->job_listing_fields();
        if(!empty($fields_wpjm)) {
          $metas = array_unique(array_merge($metas, array_keys($fields_wpjm)));
        }
      }
    }
    if(Main::isSimpleJobBoard()) {
      $job_features = maybe_unserialize(get_option('jobfeature_settings_options'));
      if(!in_array(Main::META_REMOTE_ID, $metas)) {
        $metas[] = Main::META_REMOTE_ID;
      }
      $fields_wpjm[$key] = array('label' => 'Remote ID');
      if(!empty($job_features)) {
        foreach(array('_company_name', '_company_website', '_company_tagline', '_company_logo') as $key) {
          $mykey = 'simple_job_board'.$key;
          if(!in_array($mykey, $metas)) {
            $metas[] = $mykey;
          }
        }
        foreach($job_features as $key => $vals) {
          if(is_array($vals) && strpos($key, 'jobfeature_') === 0) {
            $metas[] = $key;
            $fields_wpjm[$key] = $vals;
          }
        }
      }
    }
    return array($metas, $fields_wpjm);
  }
  static protected function meta_options($selected, $metas, $fields_wpjm, $show_invalid = true) {
    $str = '';
    $has_selected = false;
    foreach($metas as $key) {
      $label = isset($fields_wpjm[$key]['label']) ? $fields_wpjm[$key]['label'] : ucfirst(trim(str_replace('_', ' ', $key)));
      $is_selected = $selected === $key;
      $str .= '<option value="'.$key.'"'.($is_selected ? ' selected="selected"' : '').'>'.$label.'</option>';
      if($is_selected) {
        $has_selected = true;
      }
    }
    if($has_selected === false && !empty($selected)) {
      if(!$show_invalid) {
        return false;
      }
      $str .= '<option value="'.$selected.'" selected="selected">'.ucfirst(trim(str_replace('_', ' ', $selected))).' '.esc_html__('(not active)', 'wp-job-manager-api2').'</option>';
    }
    return $str;
  }
  static protected function registerWPJBAjax() {
    add_action('wp_ajax_'.Main::ACTION_PREFIX.'_export_settings', function () {
      $out = array(
        '_export_date' => gmdate('Y-m-d H:i:s').' GMT',
        '_server_name' => $_SERVER['SERVER_NAME'],
      );
      foreach(Main::getAllOptionConstants() as $option_constant) {
        $option_name = constant('com\nestdesign\WpJobManagerApi2\Main::'. $option_constant);
        $out[$option_constant] = get_option($option_name);
      }
      $custom_tax = get_option(Main::OPTION_CREATE_TAXONOMY, array());
      if(!empty($custom_tax)) {
        foreach($custom_tax as $def) {
          if(!empty($def[Main::KEY_TAXONOMY_NAME])) {
            $out['taxonomies'][$def[Main::KEY_TAXONOMY_NAME]] = self::loadTaxonomyExport($def[Main::KEY_TAXONOMY_NAME]);
          }
        }
      }
      $custom_tax = get_option(Main::OPTION_META_CUSTOM, array());
      if(!empty($custom_tax)) {
        foreach($custom_tax as $def) {
          $tax = self::meta_taxonomy_name($def);
          if(!empty($tax)) {
            $out['taxonomies'][$tax] = self::loadTaxonomyExport($tax);
          }
        }
      }
      Header('Content-Disposition: attachment; filename="wpjb-api2-'.preg_replace('/[^a-zA-Z0-9\.]+/', '-', $_SERVER['SERVER_NAME']).'-'.gmdate('Ymd').'.json"');
      die(json_encode(self::array_filter_deep($out), JSON_PRETTY_PRINT));
    });
    add_action('wp_ajax_'.Main::ACTION_PREFIX.'_import_settings', function () {
      if(!check_ajax_referer(Main::ACTION_PREFIX.'_import_settings', false, false)) {
        http_response_code(400);
        die(json_encode(array('result' => 'error', 'error' => 'Validation failed')));
      }
      if(!isset($_FILES['file']['tmp_name'])) {
        http_response_code(400);
        die(json_encode(array('result' => 'error', 'error' => 'No file uploaded')));
      }
      $fn = $_FILES['file']['tmp_name'];
      if(!is_uploaded_file($fn)) {
        http_response_code(400);
        die(json_encode(array('result' => 'error', 'error' => 'File upload failed')));
      }
      $json = json_decode(file_get_contents($fn), true);
      foreach(Main::getAllOptionConstants() as $option_constant) {
        if(isset($json[$option_constant])) {
          $option_name = constant('com\nestdesign\WpJobManagerApi2\Main::'. $option_constant);
          update_option($option_name, $json[$option_constant]);
          //TEST: Add ability to import taxonomy values for custom taxonomies
          if($option_name === Main::OPTION_CREATE_TAXONOMY && !empty($json[$option_constant])) {
            foreach($json[$option_constant] as $def) {
              if(!empty($def[Main::KEY_TAXONOMY_NAME]) && !empty($json['taxonomies'][$def[Main::KEY_TAXONOMY_NAME]])) {
                self::import_terms($def[Main::KEY_TAXONOMY_NAME], $json['taxonomies'][$def[Main::KEY_TAXONOMY_NAME]]);
              }
            }
          }
          if($option_name === Main::OPTION_META_CUSTOM && !empty($json[$option_constant])) {
            foreach($json[$option_constant] as $def) {
              $tax = self::meta_taxonomy_name($def);
              if(!empty($tax) && !empty($json['taxonomies'][$tax])) {
                self::import_terms($tax, $json['taxonomies'][$tax]);
              }
            }
          }
        }
      }
      die(json_encode(array('result' => 'success', 'redir' => 'edit.php?post_type=job_listing&page=job-manager-settings&'.Main::ACTION_PREFIX.'_imported=1')));
    });
    if(!empty($_REQUEST[Main::ACTION_PREFIX.'_imported'])) {
      add_action('admin_notices', function () {
        echo '<div class="notice notice-success is-dismissible"><p>'.__( 'The API settings was imported', 'wp-job-manager-api2' ).'</p></div>';
      });
    }
  }
  static protected function loadTaxonomyExport($taxonomy) {
    $terms = get_terms(array('taxonomy' => $taxonomy, 'hide_empty' => false));
    $out = array();
    foreach($terms as $term) {
      $item = array_intersect_key(get_object_vars($term), array_flip(array('term_id', 'name', 'parent', 'slug')));
      $item['meta'] = get_term_meta($term->term_id);
      $out[] = $item;
    }
    return $out;
  }
  static protected function addAdminJavascript() {
    add_action( 'admin_enqueue_scripts', function() {
      wp_enqueue_script( Main::ACTION_PREFIX.'_settings_js', plugin_dir_url( dirname(__FILE__ )) . '/js/functions.js', array('jquery') );
      wp_enqueue_style( Main::ACTION_PREFIX.'_settings_css', plugin_dir_url( dirname(__FILE__ )) . '/styles/admin.css', false, '1.0.0' );
    } );
  }
  //Helper functions for settings
  static protected function get_input_fields($name_prefix, $fields, $copyable = true) {
    $input_first = '<input type="text" name="'.$name_prefix.'[fields][]" class="'.($copyable ? 'copyable' : '').'" placeholder="'.esc_html__('Type in input key', 'wp-job-manager-api2').'" value="';
    return '<span class="wpjbapi2-container container-fields">'.__('Input field names', 'wp-job-manager-api2').$input_first.implode('" />'.$input_first, array_map('esc_attr', $fields)).'" /></span>';
  }
  static protected function get_multiple_meta_fields($name_prefix, $values, $metas, $meta_names, $title) {
    $str = '';
    if(is_array($values)) {
      $str .= '<div class="wpjbapi2-container">'.$title;
      foreach($values as $itm) {
        $opts = self::meta_options($itm, $metas, $meta_names, false);
        if($opts) {
          $str .= '<select name="'.$name_prefix.'[]" class="copyable"><option value="">'.esc_html__('-- select --', 'wp-job-manager-api2').'</option>';
          $str .= $opts;
          $str .= '</select>';
        }
      }
      $str .= '</div>';
    }
    return $str;
  }
  static protected function get_file_custom_meta_inputs($name_prefix, $data, $title) {
    $str = '<div class="wpjbapi2-container enabled-parent">';
    $str .='<label><input type="checkbox" class="field-enabled" name="'.$name_prefix.'[enabled]"'.(!empty($data['enabled']) ? ' checked="checked"' : '').' />'.htmlspecialchars($title).'</label>';
    $str .= '<div class="metakey-details enabled-content"><ul>';
    $str .= '<li><label><input type="text" name="'.$name_prefix.'['.Main::KEY_META_KEY.']" value="'.(!empty($data[Main::KEY_META_KEY]) ? htmlspecialchars($data[Main::KEY_META_KEY], ENT_QUOTES) : '').'" placeholder="'.htmlspecialchars(__('Meta Key, should start with _', 'wp-job-manager-api2'), ENT_QUOTES).'"></label></li>';
    $str .= '<li><label><input type="text" name="'.$name_prefix.'['.Main::KEY_SINGULAR.']" value="'.(!empty($data[Main::KEY_SINGULAR]) ? htmlspecialchars($data[Main::KEY_SINGULAR], ENT_QUOTES) : '').'" placeholder="'.htmlspecialchars(__('Meta Title', 'wp-job-manager-api2'), ENT_QUOTES).'"></label></li>';
    $str .= '</ul></div>';
    $str .='</div>';

    return $str;
  }
  static protected function checkbox_valid_required($def, $name_prefix, $with_default = false) {
    $another = '';
    $orig_js = '';
    if($with_default) {
      $another = '<label><input type="checkbox" class="field-'.Main::KEY_DEFAULT_ENABLED.'" name="'.$name_prefix.'['.Main::KEY_DEFAULT_ENABLED.']" value="1" '.(is_array($def) && !empty($def[Main::KEY_DEFAULT_ENABLED]) ? ' checked="checked"' : '').' />'.esc_html__('Use Default', 'wp-job_manager-api2').'</label>';
      $default_value_title = esc_html__('The default value to be set if none was supplied', 'wp-job-manager-api2');
      $default_value_style = ' style="'.(empty($def[Main::KEY_DEFAULT_ENABLED]) ? 'display: none;':'').'"';
      if(!empty($def[Main::KEY_CREATE_TAXONOMY])) {
        //We should create a dropdown here!
        $categories = get_categories(array('taxonomy' => self::meta_taxonomy_name($def), 'hide_empty' => false));
        if(!empty($categories)) {
          $another .= '<select'.$default_value_style.' class="field-'.Main::KEY_DEFAULT_VALUE.'" name="'.$name_prefix.'['.Main::KEY_DEFAULT_VALUE.']" title="'.$default_value_title.'"><option value="">'.esc_html__('-- select --', 'wp-job-manager-api2').'</option>'.implode('', array_map(function ($o) use($def) {
            return '<option value="'.$o->slug.'"'.(!empty($def[Main::KEY_DEFAULT_VALUE]) && $def[Main::KEY_DEFAULT_VALUE] === $o->slug ? ' selected="selected"' : '').'>'.$o->name.'</option>';
          }, $categories)).'</select>';
        } else {
          $another .= '<span'.$default_value_style.' class="field-'.Main::KEY_DEFAULT_VALUE.'">'.esc_html__('There are no terms setup', 'wp-job-manager-api2').'</span>';
        }
      } else {
        $another .= '<input'.$default_value_style.' type="text" class="field-'.Main::KEY_DEFAULT_VALUE.'" name="'.$name_prefix.'['.Main::KEY_DEFAULT_VALUE.']" value="'.(!empty($def[Main::KEY_DEFAULT_VALUE]) ? esc_attr($def[Main::KEY_DEFAULT_VALUE]) : '').'" placeholder="'.esc_html__('Default value', 'wp-job-manager-api2').'" title="'.$default_value_title.'" />';
      }
    }
    return '<span class="wpjbapi2-container container-'.Main::KEY_VALID_REQUIRED.'"><label><input type="checkbox" class="field-'.Main::KEY_VALID_REQUIRED.'" name="'.$name_prefix.'['.Main::KEY_VALID_REQUIRED.']" value="1" '.(is_array($def) && !empty($def[Main::KEY_VALID_REQUIRED]) ? ' checked="checked"' : '').' />'.esc_html__('Require value', 'wp-job_manager-api2').'</label>'.$another.'</span>';
  }
  static protected function taxonomy_default_fields($def, $name_prefix) {
    echo '<ul class="taxonomy-details">';
    echo '<li class="itm-'.Main::KEY_ENABLE_CREATE_TERM.' type-checkbox wpjbapi2-container '.(is_array($def) && !empty($def[Main::KEY_ENABLE_CREATE_TERM]) ? ' enabled' : '').'"><label><input type="checkbox" class="field-'.Main::KEY_ENABLE_CREATE_TERM.'" name="'.$name_prefix.'['.Main::KEY_ENABLE_CREATE_TERM.']" value="1" '.(is_array($def) && !empty($def[Main::KEY_ENABLE_CREATE_TERM]) ? ' checked="checked"' : '').'/>'.esc_html__('Enable create terms', 'wp-job_manager-api2').'</label></li>';
    echo '<li class="itm-'.Main::KEY_TAXONOMY_SEARCH_NAME.' type-checkbox wpjbapi2-container"><label><input type="checkbox" class="field-'.Main::KEY_TAXONOMY_SEARCH_NAME.'" name="'.$name_prefix.'['.Main::KEY_TAXONOMY_SEARCH_NAME.']" value="1" '.(is_array($def) && !empty($def[Main::KEY_TAXONOMY_SEARCH_NAME]) ? ' checked="checked"' : '').'/>'.esc_html__('Match by name', 'wp-job_manager-api2').'</label></li>';
    echo '<li class="itm-'.Main::KEY_ENABLE_CREATE_TERM.' type-checkbox">'.self::checkbox_valid_required($def, $name_prefix).'</li>';
    echo '<li class="itm-'.Main::KEY_SEPARATOR_ENABLE.' type-checkbox wpjbapi2-container"><label><input type="checkbox" name="'.$name_prefix.'['.Main::KEY_SEPARATOR_ENABLE.']" onchange="this.parentNode.nextSibling.style.display = this.checked ? \'initial\' : \'none\';" value="1"'.(!empty($def[Main::KEY_SEPARATOR_ENABLE]) ? ' checked="checked"' : '').' />'.esc_html__('Enable Separator', 'wp-job-manager-api2').'</label><span style="'.(empty($def[Main::KEY_SEPARATOR_ENABLE]) ? ' display: none' : '').'"><input type="text" name="'.$name_prefix.'['.Main::KEY_SEPARATOR.']" value="'.(!empty($def[Main::KEY_SEPARATOR]) ? htmlspecialchars($def[Main::KEY_SEPARATOR], ENT_QUOTES) : '').'" /></span></li>';
    echo '<li class="itm-'.Main::KEY_ENABLE_REPORTS.' type-checkbox wpjbapi2-container"><label>'.esc_html__('Show in reports', 'wp-job-manager-api2').' <select name="'.$name_prefix.'['.Main::KEY_ENABLE_REPORTS.']"><option value="">'.esc_html__('Do not show', 'wp-job-manager-api2').'</option><option value="slug"'.(!empty($def[Main::KEY_ENABLE_REPORTS]) && $def[Main::KEY_ENABLE_REPORTS] === 'slug' ? ' selected="selected"' : '').'>'.esc_html__('Show Slug', 'wp-job-manager-api2').'</option><option value="name"'.(!empty($def[Main::KEY_ENABLE_REPORTS]) && $def[Main::KEY_ENABLE_REPORTS] === 'name' ? ' selected="selected"' : '').'>'.esc_html__('Show Title', 'wp-job-manager-api2').'</option><option value="term_id"'.(!empty($def[Main::KEY_ENABLE_REPORTS]) && $def[Main::KEY_ENABLE_REPORTS] === 'term_id' ? ' selected="selected"' : '').'>'.esc_html__('Show Term ID', 'wp-job-manager-api2').'</option></select></li>';
    $fields = !empty($def[Main::KEY_FIELDS]) ? (array)$def[Main::KEY_FIELDS] : array();
    $fields[] = '';
    echo '<li>'.self::get_input_fields($name_prefix, $fields).'</li>';
    echo '</ul>';
  }
  static protected function array_filter_deep($array) {
    if(!is_array($array)) return $array;
    foreach($array as $key => &$val) {
      if(is_array($val)) {
        $val = self::array_filter_deep($val);
      }
    }
    unset($val);
    $array = array_filter($array);
    return $array;
  }
  static protected function create_taxonomy_fields($def, $name_prefix, $ignore = array()) {
    $str = '<span class="tax-definition">';
    foreach(array(
      Main::KEY_TAXONOMY_NAME => array(__('Code-friendly taxonomy name', 'wp-job-manager-api2'), __('Slug', 'wp-job-manager-api2')),
      Main::KEY_SINGULAR => array(__('Singular name for the taxonomy', 'wp-job-manager-api2'), __('Taxonomy', 'wp-job-manager-api2')),
      Main::KEY_PLURAL => array(__('Plural name for the taxonomy', 'wp-job-manager-api2'), __('Taxonomies', 'wp-job-manager-api2')),
    ) as $key => list($title, $placeholder)) {
      if(in_array($key, $ignore, true)) continue;
      $str .= '<input type="text" class="field-'.$key.'" name="'.$name_prefix.'['.$key.']" value="'.(isset($def[$key]) ? esc_attr($def[$key]) : '').'" placeholder="'.htmlspecialchars($placeholder, ENT_QUOTES).'" title="'.htmlspecialchars($title, ENT_QUOTES).'" />';
    }
    $str .= '</span>';
    return $str;
  }
  static protected function get_post_type_meta_keys($post_type) {
    global $wpdb;
    return $wpdb->get_col($wpdb->prepare("SELECT meta_key FROM $wpdb->postmeta as pm JOIN $wpdb->posts as p ON pm.post_id = p.ID WHERE p.post_type = %s GROUP BY meta_key", $post_type));
  }
  static private function child_display_fields($fields, $data) {
    foreach($fields as $key => $setting) {
      $value = isset($data[$key]) ? $data[$key] : '';
      echo '<div>';
      switch ( $setting['type'] ) {
        case 'text' :
          ?>
            <label for="<?php echo ( $key ); ?>"><?php echo $setting['label']; ?></label>
            <input class="widefat" id="<?php echo esc_attr( ( $key ) ); ?>" name="<?php echo ( $key ); ?>" type="text" value="<?php echo esc_attr( $value ); ?>" />
          <?php
        break;
        case 'number' :
          ?>
            <label for="<?php echo ( $key ); ?>"><?php echo $setting['label']; ?></label>
            <input class="widefat" id="<?php echo esc_attr( ( $key ) ); ?>" name="<?php echo ( $key ); ?>" type="number" step="<?php echo esc_attr( $setting['step'] ); ?>" min="<?php echo esc_attr( $setting['min'] ); ?>" max="<?php echo esc_attr( $setting['max'] ); ?>" value="<?php echo esc_attr( $value ); ?>" />
          <?php
        break;
        case 'checkbox':
          $setting['options'] = array(
            '0' => 'Disable',
            '1' => 'Enable',
          );
        case 'select' :
        case 'multiple_checkboxes':
        case 'yes_no_button' :
          $values = explode(',', $value);
          $options = array();
          foreach($setting['options'] as $k => $v) {
            $options[] = '<option value="'.$k.'" '.(in_array($k, $values) ? ' selected="selected"' : '').'>'.$v.'</option>';
          }
          ?>
            <label for="<?php echo ( $key ); ?>"><?php echo $setting['label']; ?></label>
            <select  class="widefat" id="<?php echo esc_attr( ( $key ) ); ?>" name="<?php echo ( $key ); ?><?php if($setting['type'] === 'multiple_checkboxes') echo '[]" multiple="multiple" size="5"'; else echo '"'; ?>><?php echo implode('', $options); ?></select>
          <?php
        break;
        case 'action':
          echo '<label for="'.$key.'">'.$setting['label'].'</label>';
          do_action((isset($setting['action_prefix']) ? $setting['action_prefix'] : '').$setting['action'], $setting, $cfg, $value);
          break;
        default:
          if(!empty($setting['renderer']) && $setting['renderer'] === 'et_builder_include_categories_option') {
            $output = '';
            $values = explode(',', $value);
            if ( !empty($setting['renderer_options']['use_terms']) ) {
                    $cats_array = get_terms( $setting['renderer_options']['term_name'] );
            } else {
                    $cats_array = get_categories( apply_filters( 'et_builder_get_categories_args', 'hide_empty=0' ) );
            }

            if ( empty( $cats_array ) ) {
                    $output = '<p>' . esc_html__( "You currently don't have any projects assigned to a category.", 'et_builder' ) . '</p>';
            }

            foreach ( $cats_array as $category ) {
                    $output .= sprintf(
                            '%4$s<label><input type="checkbox" name="et_pb_include_categories" value="%1$s"%3$s> %2$s</label><br/>',
                            esc_attr( $category->term_id ),
                            esc_html( $category->name ),
                            !empty($category->term_id) && in_array($category->term_id, $values) ? ' checked="checked"' : '',
                            "\n\t\t\t\t\t"
                    );
            }

            $output = '<div id="et_pb_include_categories">' . $output . '</div>';

            echo apply_filters( 'et_builder_include_categories_option_html', $output );
          }
          break;
      }
      echo '</div>';
    }
  }
}
Settings::init();
